/************************************************************************/
/* chara_messages.h							*/
/*                                                                      */
/* Defines all messages used in the CHARA control system along with the */
/* necessary data structures.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar  		                        */
/* Date   : Feb 2003							*/
/************************************************************************/

/*
 * NOTE.
 * 
 * All of these definitions and declarations where originally made
 * using 32 bit intel processors. Thus all the sizes are the same.
 * In early 2006 we installed the first 64 bit processor and this
 * has caused some problems. Here's the sizeof comparison:
 *
 *	Type		32 BIT		64 BIT
 * 	--------------------------------------
 * 	char 		1		1
 *	short int 	2		2
 *	long int 	4		8		***
 *	long long int 	8		8
 *	int 		4		4
 *	float 		4		4
 *	double 		8		8
 *	long double 	12		16		***
 *	Pointer 	4		8		***
 * 
 * Thus it is dangerous to use long int, long double, or pointers in messages
 * because they will not be the same length. This file was modifed accordingly
 * on Jan 04 2006.
 *
 * Note furtherore that using GCC on DOS you get the same as a 32 bit machine
 * even if it's a 64 bit machine. YES! An int under GCC on DOS is 4 bytes not
 * 2.
 *
 * We really should be using uint16_t like types, but for the time being this
 * will take too long to make all the required changes. For now, we remove
 * all references to long int and long double.
 * 
 * Another concern is that on the 64 bit machine floats and doubles must lie 
 * on a 64 bit boundary. This means some of the structures may be different
 * sizes. Therefore, all structures have been modified so that all doubles
 * come first, followed by floats, then integers then chars and so on.
 * The test.c program has also been changed so that it prints out the size of
 * all structures.
 *
 * MJI Additional note. A structure with double then int has 16 bytes on 
 * a 64 bit machine. So this is more complex than just a byte 
 * boundary thing...
 *
 * TtB: Indeed..... it also turns out we need to be careful about the length
 * of character strings as the two architechtures force different boundaries
 * on these. This is why there is the test.c program in this directory. It
 * should be run on each machine to make sure all the structures are the 
 * same length.
 *
 * Because of this, if you add a new structure make sure you add it to the
 * test.c program also and make sure it's length is the same on all types
 * of machines.
 */

#ifndef __CHARA_MESSAGES__
#define __CHARA_MESSAGES__

#ifndef NUM_SCOPES
#define NUM_SCOPES 6
#endif 

#ifndef NUM_DELAY_LINES
#define NUM_DELAY_LINES 6
#endif 

/* Make sure we know what the message structure looks like */

#ifndef __SMESSAGE__
struct smessage {
                        unsigned short int      type;
                        unsigned int            length;
                        unsigned char           *data;
                };

typedef struct smessage message_struct;
#define __SMESSAGE__
#endif

/************************************************************************/
/* STANDARD messages 0-99						*/
/************************************************************************/

#define FIRST_STANDARD_MESSAGE	0

/* 
 * The first and simplest message is a no-operation message.
 * It does nothing and has no data.
 */

#define NOOP	(FIRST_STANDARD_MESSAGE+0)

/*
 * The next message is a MESSAGE. In the server case,
 * the data will be a string which will be sent to the text output
 * stream. On the client side it will display the string somehow.
 * The data will be the string, not including the terminating NULL.
 * Note that this is not really required as it is often the same
 * as another define in cliserv.h or charaui.h and it had better
 * be the same too!
 */

#ifndef MESSAGE
#define MESSAGE (FIRST_STANDARD_MESSAGE+1) 
#endif

/*
 * A ping message has no agument and asks for a return message of the
 * type MESSAGE saying we are alive.
 */

#define PING	(FIRST_STANDARD_MESSAGE+2)

/*
 * An instruction to send a NOOP command to all connected machines.
 */

#define ALL_NOOP (FIRST_STANDARD_MESSAGE+3)

/*
 * An instruction to PING all connected machines.
 */

#define ALL_PING (FIRST_STANDARD_MESSAGE+4)

/* 
 * Ask for or send status information. In the UI casse it sends the 
 * structure below which will be a copy of the text in the status window.
 * Other applications may have a different idea about what to send.
 * These status window numbers should not be here.
 */

#define UI_STATUS   (FIRST_STANDARD_MESSAGE+5)
#ifndef STATUS_LINES
#define STATUS_LINES	9
#define STATUS_COLS	79
#endif

struct s_status {
		  char line[STATUS_LINES][STATUS_COLS+1];
		};

/*
 * Set a signal flag. If going from Client to server sets the flag, if going 
 * from server to client can set flags within the client, or do nothing,
 * depending on the client. Meant for sending signals, like STOP NOW and
 * so on.
 * Data is an unsigned char, the number of the signal from 0 to 255.
 */

#define UI_SIGNAL   (FIRST_STANDARD_MESSAGE+6)

/*
 * This message is sent to all clients when a server either dies or
 * when it is simply told to stop running.
 */

#define SERVER_STOPPING (FIRST_STANDARD_MESSAGE+7)

/*
 * This is a message that requests that the server send another message
 * to all clients. The message to be sent is the data and is of type 
 * int. The message sent can have no data.
 */

#define SEND_MESSAGE_ALL_CLIENTS (FIRST_STANDARD_MESSAGE+8)

/* Go into or out of remote mode, no data */

#define REMOTE_MODE_ON (FIRST_STANDARD_MESSAGE+9)
#define REMOTE_MODE_OFF (FIRST_STANDARD_MESSAGE+10)

#define LAST_STANDARD_MESSAGE REMOTE_MODE_OFF

/************************************************************************/
/* CLOCK messages 100-199						*/
/************************************************************************/

#define FIRST_CLOCK_MESSAGE	100

/* Client -> Server  Send FLush command, no data  */

#define	FLUSH_FIFOS		(FIRST_CLOCK_MESSAGE+0)

/* Client->Server run clear_lost_counts() function, no data */

#define	MESS_CLEAR_LOST_COUNTS	(FIRST_CLOCK_MESSAGE+1)

/* 
 * Client->Server run in_set(). Sends two bytes as argument.
 * First byte is the port number, second byte is in_or_out.
 *
 * Server->Client result of in_set(), Argument is used to return a TRUE
 * or FALSE result in a single byte message.
 */

#define	IN_SET			(FIRST_CLOCK_MESSAGE+2)

/*
 * Client->Server run get_rt_time and send back result, an rt_time structure.
 * 
 * Server->Client result of get_rt_time;
 */

#define RT_TIME			(FIRST_CLOCK_MESSAGE+3)

/*
 * Client->Server run get_time_next_second
 * 
 * Server->Client sends back the time of next second as CHARA_TIME
 */

#define TIME_NEXT_SECOND	(FIRST_CLOCK_MESSAGE+4)

/*
 * Client->Server run set_rt_time to set RT clock, data is CHARA_TIME
 */

#define SET_RT_TIME		(FIRST_CLOCK_MESSAGE+5)

/*
 * Client->Server runs sync_times()
 */

#define SYNC_TIMES		(FIRST_CLOCK_MESSAGE+6)

/*
 * CLient->Server runs call_sync_handler()
 */

#define CALL_SYNC_HANDLER	(FIRST_CLOCK_MESSAGE+7)

/* 
 * Client->Server calls set_lpt(), data single byte.
 * Server->Client resposne from server with LPT value, data single byte.
 */

#define MESS_SET_LPT		(FIRST_CLOCK_MESSAGE+8)

/*
 * Client -> Server calls get_lpt() Server then uses MESS_SET_LPT message
 * to return result.
 */

#define MESS_GET_LPT		(FIRST_CLOCK_MESSAGE+9)

/*
 * Client->Server calls set_bit_lpt() Server then uses SET_LPT message
 * to return the resulting LPT value.
 */

#define SET_LPT_BIT		(FIRST_CLOCK_MESSAGE+10)

/*
 * Client->Server calls set_bit_lpt() Server then uses SET_LPT message
 * to return the resulting LPT value.
 */

#define UNSET_LPT_BIT		(FIRST_CLOCK_MESSAGE+11)

/* Client->Server request reinitialization of the clock. */

#define REINIT_CLOCK		(FIRST_CLOCK_MESSAGE+12)

/* 
 * Server->Client resposne from server with LPT data port, data single byte.
 */

#define MESS_SET_LPT_DATA	(FIRST_CLOCK_MESSAGE+13)

/*
 * Client -> Server calls get_lpt_data() Server then uses MESS_SET_LPT_DATA
 * message to return result.
 */

#define MESS_GET_LPT_DATA	(FIRST_CLOCK_MESSAGE+14)

/* Last clock message */

#define LAST_CLOCK_MESSAGE	MESS_GET_LPT_DATA

/************************************************************************/
/* GPS based messages 200-299						*/
/************************************************************************/

#define FIRST_GPS_MESSAGE	200

/* All Client->Server and calls routines with no arguments */

#define GPS_START		(FIRST_GPS_MESSAGE+0)
#define GPS_STOP		(FIRST_GPS_MESSAGE+1)
#define GPS_SID			(FIRST_GPS_MESSAGE+2)
#define GPS_SYNC		(FIRST_GPS_MESSAGE+3)

/* 
 * Client->Server requesting a sync operation on a particular process 
 *  Data is the name of the process
 */

#define GPS_MSYNC		(FIRST_GPS_MESSAGE+4)

/*
 * Client->Server, request current UT and Siderial time. No data.
 * Server->Client, send back current sidereal time, as structur. 2PI=24hrs.
 */

struct s_ut_sid
        {
                double  ut;
                double  sid;
        };

#define GPS_UT_SID              (FIRST_GPS_MESSAGE+5)

#define LAST_GPS_MESSAGE	GPS_UT_STD

/************************************************************************/
/* IRIMAGE messages 300-399						*/
/************************************************************************/

#define FIRST_IRIMAGE_MESSAGE 300

/* Client->Server Set bandpass frequency */

struct s_bp_filter {
			float freq;
			float width;
			unsigned char channel; /* H_BAND or K_BAND */
		   };

#define IRIMAGE_SET_BP_FILTER		(FIRST_IRIMAGE_MESSAGE+0)

/* Client->Server Set low pass frequency */

struct s_lp_filter {
			float freq;
			unsigned char channel; /* H_BAND or K_BAND */
		   };
			
#define IRIMAGE_SET_LP_FILTER		(FIRST_IRIMAGE_MESSAGE+1)

/* client->Server Set NIRO sample time in microseconds , data is an integer */

#define IRIMAGE_SET_NIRO_SAMPLE		(FIRST_IRIMAGE_MESSAGE+2)

/* Client->Server Set NIRO wait time in microseconds , data is an integer */

#define IRIMAGE_SET_NIRO_WAIT		(FIRST_IRIMAGE_MESSAGE+3)

/* 
 * Client->Server toggle dither ON/OFF no data 
 */

#define IRIMAGE_TOGGLE_DITHER		(FIRST_IRIMAGE_MESSAGE+4)

/* Client->Server Set new scan dither, data as per structue below */

struct s_scan_dither {
			float	lambda;	/* um */
			float	freq;	/* Hz */
			float	f6db;	/* Hz */
		    };

#define IRIMAGE_SCAN_DITHER		(FIRST_IRIMAGE_MESSAGE+5)

/* Clilent->Server Set NIRO mode, argument an integer, one of */

#define NIRO_NORM               0
#define NIRO_SCAN               1
#define NIRO_ASYN               2
#define NIRO_PICT               3
#define NIRO_SYNC               4

#define IRIMAGE_NIRO_MODE		(FIRST_IRIMAGE_MESSAGE+6)

/*
 * Client->Server, request current NIRO mode, no data
 * Server->Client, result of NIRO mode request, data integer
 */

#define IRIMAGE_GET_NIRO_MODE		(FIRST_IRIMAGE_MESSAGE+7)

/* Client->Server, request NIRO resync */

#define IRIMAGE_SYNC_NIRO		(FIRST_IRIMAGE_MESSAGE+8)

/* 
 * Client->Server, request data collection.
 * Data type depends on current NIRO mode.
 * For PICT mode the data is
 */

struct s_data_pict {
			float memory; /* 0.0 to 1.0 */
			int display_mode; /* LIN, LOG, EXP */
			char	filename[256]; /* Zero length for none */
			char	display[256];  /* Must be something */
		   };

/* For NORM mode the data is */

struct s_data_norm {
			float	minsnr; /* Default 0.0 */
			int	width;	/* Default 50 */
			char	skip_save; /* Default FALSE */
			char	skip_display; /* Default FALSE */
			char	display_non_interp; /* Default FALSE */
			char	skip_low_weight; /* Save only good scans? */
			char	use_target_memory; /* Guess? */
			char	display[256];  /* Must be something */
			char	beam5[256];	/* Which scope is beam 3? */
			char	beam6[256];	/* Which scope is beam 4? */
			char	obj_type[80];	/* Type of object */
		   };
			
#define IRIMAGE_COLLECT_DATA		(FIRST_IRIMAGE_MESSAGE+9)

/* 
 * Server->Client for sending status on PICT mode display data is structure:
 */

struct s_pict_status_internal {
			float	x;
			float	y;
			float	amp;
			float	x0;
			float	y0;
			float	fwhm;
			int	min;
			int	max;
			int	in;
			int	tot;
			      };

struct s_pict_status{
			struct s_pict_status_internal ir1;
			struct s_pict_status_internal ir2;
			float	memory;
			int	sample;
			char	show;
		    };

#define IRIMAGE_PICT_STATUS	     	(FIRST_IRIMAGE_MESSAGE+10)

/* Server->client message to stop displaying pict status */

#define IRIMAGE_PICT_STATUS_STOP     	(FIRST_IRIMAGE_MESSAGE+11)
			
/* Server-> CLIent for sending status data on NORM mode display data is */

struct	s_norm_status {
			double	position;
			float	snr_now;
			float	snr_max;
			float	snr_time;
			float	vup;
			float	vdn;
			float   pos;
			float   weight;
			float   min_weight;
			float   max_error;
			float   gain;
			float	fringetarg;
			float   huge_step; 
			float   big_step; 
			float   large_step; 
			float	small_step;
			int	scans;
			int	offset;
			char	filename[80];
			char	hold;
			char	comment[80];
			char	dither_is_on;
			char    track_servo_on;
		      };

#define IRIMAGE_NORM_STATUS		(FIRST_IRIMAGE_MESSAGE+12)
			
/* Server->client message to stop displaying norm status */

#define IRIMAGE_NORM_STATUS_STOP     	(FIRST_IRIMAGE_MESSAGE+13)
			
/* 
 * Client->Server open tracking socket argument used to be structure below 
 * but is now no data.
 */

struct s_socket_name {
			int	socket;
			char	machine[256];
		     };

/* 
 * Actually, it now get's an integer as data to say which OPLE to use.
 * or if there's no data assume it's number 1
 */

#define IRIMAGE_OPEN_TRACKING_SOCKET     (FIRST_IRIMAGE_MESSAGE+14)

/* CLient->Server for controlling the display socket, data s_socket_name */

#define IRIMAGE_OPEN_DISPLAY_SOCKET      (FIRST_IRIMAGE_MESSAGE+15)

/* CLient->Server for requesting which pixels we are using on NIRO */

#define IRIMAGE_GET_NIRO_PIXELS      	(FIRST_IRIMAGE_MESSAGE+16)

/* Either direction for controling setting NIRO pixels we use */

struct s_niro_pixels {
			int size;
			int row;
			int col1;
			int col2;
		     };

#define IRIMAGE_SET_NIRO_PIXELS		(FIRST_IRIMAGE_MESSAGE+17)

/* 
 * Client->Server request DITHER_STATE message, no data 
 * Server->Client return dither state as single byte.
 */

#define IRIMAGE_DITHER_STATE		(FIRST_IRIMAGE_MESSAGE+18)

/* 
 * Client->Server, request NIRO sample time, no data.
 * Server should respond with a IRIMAGE_SET_NIRO_SAMPLE message.
 */

#define IRIMAGE_GET_NIRO_SAMPLE		(FIRST_IRIMAGE_MESSAGE+19)

/* 
 * Client->Server, request NIRO wait time, no data.
 * Server should respond with a IRIMAGE_SET_NIRO_NREADS message.
 */

#define IRIMAGE_GET_NIRO_WAIT		(FIRST_IRIMAGE_MESSAGE+20)

/* Client->Serevr, set beam combiner, data int */

#ifndef NUM_BEAM_COMBINERS
#define NUM_BEAM_COMBINERS      8
#define BEAM_COMBINER_IRIMAGE   0
#define BEAM_COMBINER_CLASSIC   1
#define BEAM_COMBINER_CLIMB     2
#define BEAM_COMBINER_FLUOR     3
#define BEAM_COMBINER_MIRC      4
#define BEAM_COMBINER_CHAMP     5
#define BEAM_COMBINER_PAVO      5
#define BEAM_COMBINER_VEGA      7
#endif

#define IRIMAGE_SET_BEAM_COMBINER	(FIRST_IRIMAGE_MESSAGE+21)

/* Client->Server for controlling what kind of PS we see */

#define IRIMAGE_PS_UNFITLERED		(FIRST_IRIMAGE_MESSAGE+22)

#define IRIMAGE_PS_FITLERED		(FIRST_IRIMAGE_MESSAGE+23)

/* Client->Server Set new linear dither, data as per structue below */

struct s_linear_dither {
			int	length;	/* steps */
			int	inc;	/* incriment */
		    };

#define IRIMAGE_LINEAR_DITHER		(FIRST_IRIMAGE_MESSAGE+24)

/* Client->Server Set new phase dither, data integer number of steps */

#define IRIMAGE_PHASE_DITHER		(FIRST_IRIMAGE_MESSAGE+25)

#define IRIMAGE_RESET_FILTER		(FIRST_IRIMAGE_MESSAGE+26)

/* CLient->Server for controlling the niro socket, data s_socket_name */

#define IRIMAGE_OPEN_NIRO_SOCKET      (FIRST_IRIMAGE_MESSAGE+27)

/* Client->Server for asking for power of two for number of reads, no data */

#define IRIMAGE_GET_NIRO_NREADS	      (FIRST_IRIMAGE_MESSAGE+28)

/*
 * Client->Server for setting number of reads, data struct
 * Server->Client response to IRIMAGE_GET_NIRO_NREADS, data struct
 */
 
struct s_niro_nreads {
			int	nreads;
			int	nloops;
			int	nskips;
			int	nboundary;
		    };

#define IRIMAGE_SET_NIRO_NREADS       (FIRST_IRIMAGE_MESSAGE+29)

/* Client->Server request mean count calculation. Data integer samples */

#define IRIMAGE_GET_MEAN_COUNTS		(FIRST_IRIMAGE_MESSAGE+30)

/* Client->Server for setting a standard NIRO type. Data integer, one of */

#define NIRO_UNKNOWN		-1
#define NIRO_1000HzNPIX2	0
#define NIRO_750HzNPIX2		1
#define NIRO_500HzNPIX2		2
#define NIRO_250HzNPIX2		3
#define NIRO_1000HzNPIX1	4
#define NIRO_750HzNPIX1		5
#define NIRO_500HzNPIX1		6
#define NIRO_250HzNPIX1		7
#define NUM_NIRO_STANDARD_TYPES	8

#define IRIMAGE_SET_NIRO_STANDARD_TYPE	(FIRST_IRIMAGE_MESSAGE+31)

/* Client->Server for changing the tracking mode Data integer and 
 * one of the following
 */

#ifndef FRINGE_SCAN
#define FRINGE_SCAN              	0 /* Use long scans */
#define FRINGE_PHASE             	1 /* Tracking to phase */
#endif

#define IRIMAGE_SET_FRINGE_MODE		(FIRST_IRIMAGE_MESSAGE+32)

#define IRIMAGE_ARM_NIRO		(FIRST_IRIMAGE_MESSAGE+33)

#define IRIMAGE_SHOW_PS			(FIRST_IRIMAGE_MESSAGE+34)
#define IRIMAGE_SHOW_ENV		(FIRST_IRIMAGE_MESSAGE+35)

/* Client->Server for changing the number of data scans, data struct */

struct s_set_data_scans {
	int data_scans;
	int shutter_scans_a;
	int shutter_scans_b;
		        };

#define IRIMAGE_SET_DATA_SCANS		(FIRST_IRIMAGE_MESSAGE+36)
                                                                                
/* 
 * Client->Server Setting fast dither on or off, data unsigned char.
 * Server->Client MEssage to inform client of fast dither state, unsigned char
 */

#define IRIMAGE_SET_FAST_DITHER		(FIRST_IRIMAGE_MESSAGE+37)

/* Client->Server, request server to send IRIMAGE_DITHER_STATE. */

#define IRIMAGE_FAST_DITHER_STATE	(FIRST_IRIMAGE_MESSAGE+38)

/* Client->Server set tracking cart, data an int. */

#define IRIMAGE_SET_TRACKING_CART	(FIRST_IRIMAGE_MESSAGE+39)

/* Client-> server set servo direction. data an int, only sign used. */

#define IRIMAGE_SET_DIRECTION		(FIRST_IRIMAGE_MESSAGE+40)

/* Client->server ask for current niro reset delay, no data */

#define IRIMAGE_GET_NIRO_RESET_DELAY	(FIRST_IRIMAGE_MESSAGE+41)

/* 
 * Server-> Client reply to previous, data integer
 * Client->Server set niro reset delay, data integer
 */

#define IRIMAGE_SET_NIRO_RESET_DELAY	(FIRST_IRIMAGE_MESSAGE+42)

/* Client->server ask for current niro reset time, no data */

#define IRIMAGE_GET_NIRO_RESET_TIME	(FIRST_IRIMAGE_MESSAGE+43)

/* 
 * Server-> Client reply to previous, data integer
 * Client->Server set niro reset time, data integer
 */

#define IRIMAGE_SET_NIRO_RESET_TIME	(FIRST_IRIMAGE_MESSAGE+44)

/* Client->Server Force dither to go ON. No data. */

#define IRIMAGE_TURN_DITHER_ON		(FIRST_IRIMAGE_MESSAGE+45)

/* Client->Server Force dither to go OFF. No data. */

#define IRIMAGE_TURN_DITHER_OFF		(FIRST_IRIMAGE_MESSAGE+46)

/* Client->Server Set new flexible dither, data as per structue below */

struct s_flexible_dither {
			float	lambda;	/* um */
			float	freq;	/* Hz */
			float	amplitude; /* um */
		    };

#define IRIMAGE_FLEXIBLE_DITHER		(FIRST_IRIMAGE_MESSAGE+47)

/* Client->Server Set feedback dither, data as for s_flexible_dither */

#define IRIMAGE_FEEDBACK_DITHER		(FIRST_IRIMAGE_MESSAGE+48)

#define LAST_IRIMAGE_MESSAGE		IRIMAGE_FEEDBACK_DITHER

/************************************************************************/
/* And now we define the signals that IRIMAGE knows about.		*/
/************************************************************************/

/* These signals are used in the PICT display routine */

#define IRIMAGE_SIGNAL_MAKE_BIAS	1
#define IRIMAGE_SIGNAL_ZERO_BIAS	2
#define IRIMAGE_SIGNAL_SHOW_PICTURE	3
#define IRIMAGE_SIGNAL_AUTOSCALE	4
#define IRIMAGE_SIGNAL_SHOW_DIFF	5
#define IRIMAGE_SIGNAL_SHOW_FIT		6

/* These signals are used in the NORM display routine */

#define IRIMAGE_SIGNAL_CLEAR		7
#define IRIMAGE_SIGNAL_SCALE_UP		8
#define IRIMAGE_SIGNAL_SCALE_DOWN	9
#define IRIMAGE_SIGNAL_RECORD		10
#define IRIMAGE_SIGNAL_HOLD		11
#define IRIMAGE_SIGNAL_SEND		12
#define IRIMAGE_SIGNAL_OFFSET_UP_1	13
#define IRIMAGE_SIGNAL_OFFSET_DOWN_1	14
#define IRIMAGE_SIGNAL_OFFSET_UP_500	15
#define IRIMAGE_SIGNAL_OFFSET_DOWN_500	16
#define IRIMAGE_SIGNAL_TOGGLE_TRACK_SERVO 17
#define IRIMAGE_SIGNAL_MIN_WEIGHT_UP	18
#define IRIMAGE_SIGNAL_MIN_WEIGHT_DOWN	19
#define IRIMAGE_SIGNAL_MAX_ERROR_DOWN	20
#define IRIMAGE_SIGNAL_MAX_ERROR_UP	21
#define IRIMAGE_SIGNAL_GAIN_DOWN	22
#define IRIMAGE_SIGNAL_GAIN_UP		23
#define IRIMAGE_SIGNAL_TOGGLE_DIRECTION 24
#define IRIMAGE_SIGNAL_TARG_UP_1	25
#define IRIMAGE_SIGNAL_TARG_DN_1	26
#define IRIMAGE_SIGNAL_TARG_UP_10	27
#define IRIMAGE_SIGNAL_TARG_DN_10	28
#define IRIMAGE_SIGNAL_POS_UP_SMALL	29
#define IRIMAGE_SIGNAL_POS_DN_SMALL	30
#define IRIMAGE_SIGNAL_POS_UP_LARGE	31
#define IRIMAGE_SIGNAL_POS_DN_LARGE	32
#define IRIMAGE_SIGNAL_SIG_DIFF		33
#define IRIMAGE_SIGNAL_STOP		34
#define IRIMAGE_SIGNAL_POS_UP_BIG	35
#define IRIMAGE_SIGNAL_POS_DN_BIG	36
#define IRIMAGE_SIGNAL_POS_UP_HUGE	37
#define IRIMAGE_SIGNAL_POS_DN_HUGE	38

/* This is used in several places */

#define IRIMAGE_SIGNAL_EXIT		50

/************************************************************************/
/* OPLE messages 400-499						*/
/************************************************************************/

#define FIRST_OPLE_MESSAGE		400

/* Client->Server Open ople socket, no data */

#define OPLE_OPEN_SOCKET		(FIRST_OPLE_MESSAGE+0)

/* Client->Server Close ople socket, no data */

#define OPLE_CLOSE_SOCKET		(FIRST_OPLE_MESSAGE+1)

/* 
 * Client-> Server select a target structure and data 
 * Server->Client response to what it just did
 */

struct s_select_target {
				int dl_number; 
				int target; 
				char set_active; 
		       };

#define OPLE_SELECT_TARGET		(FIRST_OPLE_MESSAGE+2)

/* 
 * Client->Server Set manual and oplen loop targets, data as in struct 
 * Note, the unsigned int should be OPLE_TIME. 
 * This is not cool but it works.
 */


struct s_target {
			double position; 
			double rate;
			int dl_number; 
			unsigned int time;
		};

#define OPLE_OPEN_LOOP_TARGET		(FIRST_OPLE_MESSAGE+3)
#define OPLE_MANUAL_TARGET		(FIRST_OPLE_MESSAGE+4)

/* Client->Server command to turn cart off, data is an int */

#define OPLE_CART_OFF			(FIRST_OPLE_MESSAGE+5)

/* Client->Server command to turn all carts off, no data */

#define OPLE_ALL_CARTS_OFF		(FIRST_OPLE_MESSAGE+6)

/* Client->Server command to send cart front, data is an int */

#define OPLE_CART_FRONT			(FIRST_OPLE_MESSAGE+7)

/* Client->Server command to send cart back, data is an int */

#define OPLE_CART_BACK			(FIRST_OPLE_MESSAGE+8)

/* Client->Server command to send cart track, data is an int */

#define OPLE_CART_TRACK			(FIRST_OPLE_MESSAGE+9)

/* Client->Server command to send cart home, data is an int */

#define OPLE_CART_HOME			(FIRST_OPLE_MESSAGE+10)

/* Client->Server command to clear cart laser, data is an int */

#define OPLE_CART_CLEAR_LASER		(FIRST_OPLE_MESSAGE+11)

/* Client->Server command to start astromod, no data */

#define OPLE_START_ASTROMOD		(FIRST_OPLE_MESSAGE+12)

/* Client->Server command to stop astromod, no data */

#define OPLE_STOP_ASTROMOD		(FIRST_OPLE_MESSAGE+13)

/* Client->Server command to set reference, data as in struct */

struct s_set_reference {
				double position;
				double rate;
		       };

#define OPLE_SET_REFERENCE_CART		(FIRST_OPLE_MESSAGE+14)

/* Client->Server request return messages of form SLECT_TARGET so
	client knows what's up */

#define OPLE_CURRENT_SELECT_TARGETS	(FIRST_OPLE_MESSAGE+15)

/* 
 * Client->Sever Request current manual target, no data.
 * Server->CLient Send current manual target, data is array of doubles.
 * Units are in meters.
 */

#define OPLE_CURRENT_MANUAL_TARGETS	(FIRST_OPLE_MESSAGE+16)

/* 
 * Client->Sever Request current open_loop targets, no data.
 * Server->CLient Send current open_loop targets, data is array of doubles.
 * Units are in meters.
 */

#define OPLE_CURRENT_OPEN_LOOP_TARGETS (FIRST_OPLE_MESSAGE+17)

/*
 * Client->Server Start scaning, data in structure determines
 * scan range frequency and so on. ANy client getting this message knows
 * not to try and do anything else for a while, at least it knows to pop
 * up the right window.
 * The STOP_SCAN message tells all the clients we are done.
 * Server->Client (mostly stop) to tell clients that we haved stopped.
 */

struct s_start_scan {
		double	start;
		double	stop;
		double	inc;
		double	lambda;
		double	freq;
		int	dl;
		int	scans;
	      };

#define	OPLE_START_SCAN			  (FIRST_OPLE_MESSAGE+18)
#define	OPLE_STOP_SCAN			  (FIRST_OPLE_MESSAGE+19)

/* 
 * Server-> Client Sends sscan status data. If the client get's this it then
 * knows that we're scanning
 */

struct s_scan_status {
		double	start;	 /* cm */
		double	stop;	 /* cm */
		double	current_position; /* cm */
		double	vel; /* um/S */
		double	scan_time_left;	/* Secs */
		int	dl;	/* cart */
		int	filenumber; /* Integer */
		int	current_scan;
		int	number_of_scans;
		double  cart_errors[NUM_DELAY_LINES];
		     };
		
#define	OPLE_SCAN_STATUS		  (FIRST_OPLE_MESSAGE+20)

/* Client->Server Request to toggle metrology data, data integer cart number */

#define OPLE_METROLOGY_DATA_ON		  (FIRST_OPLE_MESSAGE+21)
#define OPLE_METROLOGY_DATA_OFF		  (FIRST_OPLE_MESSAGE+22)

/* Client->Server Request to sync time on ople rack, no data */

#define OPLE_SYNC_OPLE			  (FIRST_OPLE_MESSAGE+23)

/* Client->Server open/close log file, no data */

#define OPLE_OPEN_LOG			(FIRST_OPLE_MESSAGE+24)
#define OPLE_CLOSE_LOG			(FIRST_OPLE_MESSAGE+25)

/* 
 * Client->Server request sending same message to all clients, data is
 * an int to say which cart we wish to offset.
 * Server->Client Request opening offset window on client, data is
 * an int to say which cart we wish to offset.
 */

#define OPLE_RUN_OFFSET			(FIRST_OPLE_MESSAGE+26)

/*
 * Client->Server, request tracking errors, no data.
 * Server->Client, send tracking errors, data 2 arrays 
 * of floats[NUM_DELAY_LINES]. First cart positions, second cart errors.
 */

#define OPLE_TRACKING_ERROR		(FIRST_OPLE_MESSAGE+27)

/* 
 * Client->Server request for return message of same type, no data
 * Server->Client response. Data single char TRUE if astromod running,
 *		FALSE otherwise
 */

#define OPLE_ASTROMOD_RUNNING          (FIRST_OPLE_MESSAGE+28)

/* Message to set automatic reference on OPLE, data is a char TRUE or FALSE */

#define OPLE_AUTO_REFERENCE		(FIRST_OPLE_MESSAGE+29)

/*
 * Client->Server, request current status of limit and home switches.
 * Server->Client response. Data array[NUM_DELAY_LINES] of structures as below
 */

#define OPLE_LIMITS_STATUS		(FIRST_OPLE_MESSAGE+30)

struct s_limits_status {
				char	hard_front;
				char	soft_front;
				char	home;
				char	soft_back;
				char	hard_back;
			};

/*
 * Client->Server Request for message of same type, no data.
 * Server->Client Response, returns char array telling if each cart is
 * moving or not. TRUE means the cart is moving.
 */

#define OPLE_CARTS_MOVING		(FIRST_OPLE_MESSAGE+31)

/* 
 * Client->Server set new manual target offsets for all six scopes.
 * Data is array of NUM_SCOPE doubles with new offsets. Rates assumed
 * to be zero.
 */

#define OPLE_MANUAL_OFFSETS		(FIRST_OPLE_MESSAGE+32)

/* 
 * Client->Server, set active carts data char[NUM_DELAY_LINES]
 * Server->Client, response to OPLE_GET_ACTIVE_CARTS, same data.
 */

#define OPLE_SET_ACTIVE_CARTS		(FIRST_OPLE_MESSAGE+33)

/* Client->Server request for which carts are active. */

#define OPLE_GET_ACTIVE_CARTS		(FIRST_OPLE_MESSAGE+34)

/* Client->Server . NB this HAS to be a float because of the
 * weird 64 bit things. 
 * Server->Client - sends back the total offset.
 */

struct s_air_offset {
		float offset; 
  		int dl_number; 
		    };

#define OPLE_AIR_PATH_OFFSET_MESSAGE	(FIRST_OPLE_MESSAGE+35)

/* Client server: Invokes call_optimized_reference_position() no data. */

#define OPLE_CALL_OPTIMIZED_REFERENCE_POSITION (FIRST_OPLE_MESSAGE+36)

/* Client server: Invokes set_optimized_reference_position() no data. */

#define OPLE_SET_OPTIMIZED_REFERENCE_POSITION (FIRST_OPLE_MESSAGE+37)

/* 
 * Client server: Requests saving baseline solution data. 
 * If no data given will use current manual offsets in software.
 * If an array of doubles is there it will use the numbers in that
 * array instead. Numbers in array should be in meters and in terms of
 * cart position not delay.
 */

#define OPLE_BASELINE_SOLUTION_DATA     (FIRST_OPLE_MESSAGE+38)

/*
 * Client->Server, ask for a specific cart's servo parameters.
 * Data is an integer defining the cart.
 */

#define OPLE_GET_SERVO_PARAMETERS	(FIRST_OPLE_MESSAGE+39)

/*
 * Both. Sends a structure of the type below whch is then either sent
 * to the VME crate by the server or used by the client.
 * This structure is very like the tGuiGsuDlServoParameters structure
 * in the PLE server, but without the spacers and including a cart number.
 */

#ifndef VME_BOOL
#define VME_BOOL int
#endif
struct s_ople_servo_parameters {
  
	    /* Servo Rates in Hz */
	    
	    double dPztServoRate;
	    double dVcServoRate;
	    double dMotorServoRate;
	 
	    /* Pzt Control Parameters */
	    
	    double dPztGain;
	    double dPztLagPole;
	    double dPztLagZero;
	 
	    /* VC 1 Control Parameters */
	    
	    double dVc1StabilizePidGain;
	    double dVc1StabilizePidZero1;
	    double dVc1StabilizePidZero2;
	    double dVc1StabilizePidPole;

	    double dVc1TrackPidGain;
	    double dVc1TrackPidZero1;
	    double dVc1TrackPidZero2;
	    double dVc1TrackPidPole;
	    double dVc1TrackLaserFeedforwardGain;
	    double dVc1TrackFilterMemory;

	    /* VC 2 Control Parameters */

	    double dVc2StabilizePidGain;
	    double dVc2StabilizePidZero1;
	    double dVc2StabilizePidZero2;
	    double dVc2StabilizePidPole;
	    double dVc2StabilizeMotorAccelerationFeedforwardGain;
	 
	    double dVc2TrackPidGain;
	    double dVc2TrackPidZero1;
	    double dVc2TrackPidZero2;
	    double dVc2TrackPidPole;
	    double dVc2TrackVc1ForceFeedforwardGain;
	   
	    /* Motor Control Parameters */
	    
	    double dMotorSlewVelocity;
	    double dMotorCreepVelocity;
	    double dDecelerationAggressiveness;            
	    double dMotorPidGain;
	    double dMotorPidZero1;
	    double dMotorPidZero2;
	    double dMotorPidPole;
	    double dMotorFeedForwardGain;
	    

	    /* Controller Mode Thresholds */

	    double dDecelToIdleVelocityThreshold;
	    double dSlewToTrackThresholdInMm;
	    double dTrackToSlewThresholdInMm;

	    /* Servo Enables */
	    
	    VME_BOOL bPztServoEnabled;
	    VME_BOOL bPztLagEnabled;
	    VME_BOOL bVc1TrackServoEnabled;
	    VME_BOOL bVc1StabilizeServoEnabled;
	    VME_BOOL bVc2TrackServoEnabled;
	    VME_BOOL bVc2StabilizeServoEnabled;
	    VME_BOOL bMotorServoEnabled;

	    /* Which cart? */

	    int  cart;
	};

#define OPLE_SET_SERVO_PARAMETERS	(FIRST_OPLE_MESSAGE+40)

/* Server->Client send current cart errors. Array of 6 doubles in microns. */

#define OPLE_CART_ERRORS		(FIRST_OPLE_MESSAGE+41)

/* Server->Client send current cart position. Array of 6 doubles in microns. */

#define OPLE_CART_POSITIONS		(FIRST_OPLE_MESSAGE+42)

/* Client->Server "I want to use the LDC (glass)" */
#define OPLE_USE_LDC		        (FIRST_OPLE_MESSAGE+43)

/* Client->Server "There is no glass in the beam - forget about it" */
#define OPLE_NO_LDC		        (FIRST_OPLE_MESSAGE+44)

/* Client->Server request glass targets */
/* Server->Client return them as an array of doubles.*/
#define OPLE_CURRENT_GLASS_TARGETS      (FIRST_OPLE_MESSAGE+45)

#define LAST_OPLE_MESSAGE               OPLE_CURRENT_GLASS_TARGETS

/************************************************************************/
/* And now we define the signals that OPLE knows about.			*/
/************************************************************************/

/* The first set of signals is for the scan mode */

#define OPLE_SIGNAL_SCAN_CENTER		1
#define OPLE_SIGNAL_SCAN_FORWARD	2
#define OPLE_SIGNAL_SCAN_BACKWARD	3
#define OPLE_SIGNAL_SCAN_INCREASE	4
#define OPLE_SIGNAL_SCAN_DECREASE	5
#define OPLE_SIGNAL_SCAN_STOP_ZERO	6
#define OPLE_SIGNAL_SCAN_STOP_HERE	7
#define OPLE_SIGNAL_SCAN_EDGE		8

/************************************************************************/
/* WOBBLE messages 500-599						*/
/************************************************************************/

#define FIRST_WOBBLE_MESSAGE	500

/*
 * Message to request BDISP. Client->Server. Data as per structure below.
 */

#define	WOBBLE_BDISP		(FIRST_WOBBLE_MESSAGE+0)

struct s_bdisp {
			int	display_length;
			char	display[256];
	       };

/*
 * CLient->Server reuests start of display_position function.
 * Data as in structure below.
 */

#define WOBBLE_CDISP		(FIRST_WOBBLE_MESSAGE+1)

struct s_cdisp {
			int	detector;
			int	length;
			char	display[256];
		};
/*
 * Server->CLient sending the cdisp status info as structure below.
 */

struct s_cdisp_status {
			float	a;
			float	b;
			float	c;
			float	d;
			float	sum;
			int	samples;
			char	name[81];
		      };

/*
 * Server->Client stop sending the cdisp status info.
 */

#define WOBBLE_CDISP_STOP	(FIRST_WOBBLE_MESSAGE+2)

/*
 * Client->Server request chanel names, no data 
 */

#define WOBBLE_NAMES           (FIRST_WOBBLE_MESSAGE+3)

/*
 * Server->Client list of names as per structure
 */

struct s_names {
		char names[2][81];
	       };

/* Client->Server - set names, data as per structure above */

#define WOBBLE_SET_NAMES	(FIRST_WOBBLE_MESSAGE+4)

/*
 * Client->server request status information on one chanel.
 * Data is an integer specifying the chanel.
 */

#define WOBBLE_STATUS		(FIRST_WOBBLE_MESSAGE+5)

/*
 * Server->Client status information, data in following structure
 */

struct s_det_status {
			int	      det;   /* Which detector */
			unsigned char mod_x; /* TRUE or FALSE */
			unsigned char mod_y; /* TRUE or FALSE */
			unsigned char on;    /* TRUE or FALSE */
			unsigned char send;  /* TRUE or FALSE... for sending */
		};

/*
 * CLient->Server turn signals on or off. No data
 */

#define WOBBLE_SIGON		(FIRST_WOBBLE_MESSAGE+6)
#define WOBBLE_SIGOFF		(FIRST_WOBBLE_MESSAGE+7)

/*
 * Client->server, for turing on/off servo, modulation and signals
 * all have integer (chanel) for arguement
 */

#define WOBBLE_ON		(FIRST_WOBBLE_MESSAGE+8)
#define WOBBLE_OFF		(FIRST_WOBBLE_MESSAGE+9)
#define WOBBLE_MODX		(FIRST_WOBBLE_MESSAGE+10)
#define WOBBLE_MODY		(FIRST_WOBBLE_MESSAGE+11)
#define WOBBLE_SENDON		(FIRST_WOBBLE_MESSAGE+12)
#define WOBBLE_SENDOFF		(FIRST_WOBBLE_MESSAGE+13)

/*
 * CLient->Server request reference frame images, data of form
 */

#define WOBBLE_REF		(FIRST_WOBBLE_MESSAGE+14)

struct s_ref {
		int	row;
		int	col;
		char	display[256];
	     };

/*
 * Server->Client for sending reference status information of form
 */

struct s_ref_status {
			float	frames_per_second;
			float	remote_frames_per_second;
			float	maximum;
			float	pos_x;
			float	pos_y;
			float 	org_x;
			float	org_y;
			float	err_x;
			float	err_y;
			float	cur_x;
			float	cur_y;
			int	frames;
			int	remote_frames;
			int	pix_x;
			int	pix_y;
			int	brightness;
			int	contrast;
			int 	zoom;
			int	mean_n;
		    };

/*
 * Server->Client for stopping reference status information 
 */

#define WOBBLE_REF_STOP	(FIRST_WOBBLE_MESSAGE+15)

/*
 * Client->Server for setting sample period, data integer period
 */

#define WOBBLE_PERIOD		(FIRST_WOBBLE_MESSAGE+16)

/*
 * Client->Server for setting varriance period, data integer period
 */

#define WOBBLE_VAR_PERIOD	(FIRST_WOBBLE_MESSAGE+17)

/*
 * Client->Server for runnign auto-tuner, no data
 */

#define WOBBLE_AUTO_TUNE	(FIRST_WOBBLE_MESSAGE+18)

/*
 * Client->Server for setting current servo parameters
 * Server->Client for sending current servo parameters
 * Both have the following data structure
 */

#define WOBBLE_SERVO_PARAMS	(FIRST_WOBBLE_MESSAGE+19)

struct	s_servo_params {
				float	prop;
				float	diff;
				int	det;
		       };

/*
 * Client->Server, request servo params, data integer chanel number
 */

#define WOBBLE_GET_SERVO_PARAMS	(FIRST_WOBBLE_MESSAGE+20)

/*
 * Client->Server, request a display socket, data in structure s_socket_name
 */

#define WOBBLE_OPEN_DISPLAY_SOCKET	(FIRST_WOBBLE_MESSAGE+21)

/*
 * Client->Server for setting maximum allowed change. Data in structure.
 * Server->Client response to request for current value, same data.
 */

#define	WOBBLE_SET_MAX_CHANGE		(FIRST_WOBBLE_MESSAGE+22)

struct s_max_change {
			float	max_change;
			int	det;
		    };

/*
 * Client->Server, requesting current max. Data integer detector number
 */

#define	WOBBLE_GET_MAX_CHANGE		(FIRST_WOBBLE_MESSAGE+23)

/* Client->Server, sets names of scopes, data as per structure */

struct s_scope_names {
			char beam5_name[3];
			char beam6_name[3];
		     };

#define WOBBLE_SCOPE_NAMES		(FIRST_WOBBLE_MESSAGE+24)

/* 
 * Client->Server, request counts seen in system, no data.
 * Server->Client, return counts seen, data in structure.
 */

struct s_counts {
			int	beam5;
			int	beam6;
		};

#define WOBBLE_COUNTS			(FIRST_WOBBLE_MESSAGE+25)

/* 
 * Client->Server, request errors seen in system, no data.
 * Server->Client, return errors seen, data in structure.
 */

struct s_errors {
			float	beam5;
			float	beam6;
		};

#define WOBBLE_ERRORS			(FIRST_WOBBLE_MESSAGE+26)

/* Client->Server, requests logging of seeing, no data */

#define WOBBLE_LOG_SEEING		(FIRST_WOBBLE_MESSAGE+27)

/* Client->Server, open or close the log file, no data */

#define WOBBLE_OPEN_LOG			(FIRST_WOBBLE_MESSAGE+28)
#define WOBBLE_CLOSE_LOG		(FIRST_WOBBLE_MESSAGE+29)

/*
 * Client->Server, request a display socket, data in structure s_socket_name
 */

#define WOBBLE_OPEN_REF_SOCKET		(FIRST_WOBBLE_MESSAGE+30)

/* Client->Server start or stop data recording */

#define WOBBLE_START_RECORD		(FIRST_WOBBLE_MESSAGE+31)
#define WOBBLE_STOP_RECORD		(FIRST_WOBBLE_MESSAGE+32)

/*
 * Client->Server request channel spec frame images, data is same as WOBBLE_REF
 */

#define WOBBLE_CHAN			(FIRST_WOBBLE_MESSAGE+33)

/*
 * Server->Client for sending reference status information of form
 */

struct s_chan_status {
			float	frames_per_second;
			float	maximum;
			int	frames;
			int	pix_x;
			int	pix_y;
			int	brightness;
			int	contrast;
		    };

/*
 * Server->Client for stopping reference status information 
 */

#define WOBBLE_CHAN_STOP	(FIRST_WOBBLE_MESSAGE+34)

/*
 * Client->Server, request seeing loig filename. NO data
 * Server->Client, Send seeing log filename, data filename including terminating
 * null.
 */

#define WOBBLE_LOG_FILENAME		(FIRST_WOBBLE_MESSAGE+35)

#define LAST_WOBBLE_MESSAGE		WOBBLE_LOG_FILENAME

/************************************************************************/
/* signals, mostly used for within functions.				*/
/************************************************************************/

#define WOBBLE_SIGNAL_STOP		1
#define WOBBLE_SIGNAL_CLEAR		2
#define WOBBLE_SIGNAL_BRIGHT_UP		3
#define WOBBLE_SIGNAL_BRIGHT_DOWN	4
#define WOBBLE_SIGNAL_CONTRAST_UP	5
#define WOBBLE_SIGNAL_CONTRAST_DOWN	6
#define WOBBLE_SIGNAL_REVERSE_COLOR	7
#define WOBBLE_SIGNAL_SET_ORIGIN	8
#define WOBBLE_SIGNAL_ZOOM_IN		9
#define WOBBLE_SIGNAL_ZOOM_OUT		10
#define WOBBLE_SIGNAL_CROSS_HAIR	11
#define WOBBLE_SIGNAL_AUTOALIGN		12
#define WOBBLE_SIGNAL_UNDOALIGN		13
#define WOBBLE_SIGNAL_BIAS		14
#define WOBBLE_SIGNAL_BLUE_LINE		15
#define WOBBLE_SIGNAL_SAVE		16
#define WOBBLE_SIGNAL_MEAN_UP		17
#define WOBBLE_SIGNAL_MEAN_DOWN		18
#define WOBBLE_SIGNAL_UP		19
#define WOBBLE_SIGNAL_DOWN		20
#define WOBBLE_SIGNAL_LEFT		21
#define WOBBLE_SIGNAL_RIGHT		22

/************************************************************************/
/* ESP messages 600-699							*/
/************************************************************************/

#define FIRST_ESP_MESSAGE	600

/* Structure for axis information */

struct AXIS{
                double position;
                double relative_step_size;
                double real_position;
                double velocity;
                double acceleration;
                float min;
                float max;
                int num;
                int beam;
                int power;
                int motion_done;
           };

/* 
 * Message(s): ESP_ON/ESP_OFF
 * Direction: Client->Server
 * Data:  struct AXIS
 * Description: Set the "num" to the axis number.  Requests server to turn
 * power to axis "num" on or off.  Triggers server bg process to send power
 * status message (ESP_POWER_STATUS)
 */
#define ESP_ON			(FIRST_ESP_MESSAGE+0)
#define ESP_OFF			(FIRST_ESP_MESSAGE+1)

/* 
 * Message(s): ESP_GET_STATUS
 * Direction: Client->Server
 * Data:  struct AXIS
 * Description:  NOT USED!!  Maybe it should replace all the other status
 * requests.  
 */
#define ESP_GET_STATUS		(FIRST_ESP_MESSAGE+2)

/* 
 * Message(s): ESP_STATUS
 * Direction: Server->Client
 * Data:  struct AXIS
 * Description:  NOT USED!!  Maybe it should replace all the other status
 * requests.  
 */
#define ESP_STATUS		(FIRST_ESP_MESSAGE+3)

/* 
 * Message(s): ESP_GET_NAMES
 * Direction: Client->Server
 * Data:  NULL
 * Description:  Sends request for names of all of the axes.  Triggers server
 * to send the names back to the client in message ESP_NAMES.
 */
#define ESP_GET_NAMES		(FIRST_ESP_MESSAGE+4)

/* 
 * Message(s): ESP_NAMES
 * Direction: Server->Client
 * Data:  char string
 * Description:  Server sends the unit name and axes names in one string.  It's
 * up to the user to chop it up.  See an example in gtkclient/esp_messages.c
 */
#define ESP_NAMES		(FIRST_ESP_MESSAGE+5)


/* 
 * Message(s): ESP_MOVE_ABS
 * Direction: Client->Server
 * Data:  struct AXIS
 * Description:  Set the "num" to the axis number.  Requests server to move
 * axis to an absolute position (units depend on controller but most often it's
 * millimeters).  ESP_MOTION_DONE_STATUS will be sent from the server to the
 * client when it's done moving followed by a ESP_POSITION message reporting
 * the current position for that axis.
 */
#define ESP_MOVE_ABS		(FIRST_ESP_MESSAGE+6)

/* 
 * Message(s): ESP_MOVE_HOME
 * Direction: Client->Server
 * Data:  struct AXIS
 * Description:  Set the "num" to the axis number.  Requests server to move
 * axis to home position.  ESP_MOTION_DONE_STATUS will be sent from the server
 * to the client when it's done moving followed by a ESP_POSITION message
 * reporting the current position for that axis.
 */
#define ESP_MOVE_HOME		(FIRST_ESP_MESSAGE+7)

/* 
 * Message(s): ESP_TEXT
 * Direction: Client->Server
 * Data:  char string
 * Description:  Send a character string command to the server.
 * ESP_MOTION_DONE_STATUS will be sent from the server to the client when it's
 * done moving followed by a ESP_POSITION message reporting the current
 * position for that axis. 
 */
#define ESP_TEXT		(FIRST_ESP_MESSAGE+8)

/* 
 * Message(s): ESP_MOVE_REL
 * Direction: Client->Server
 * Data:  struct AXIS
 * Description:  Set the "num" to the axis number.  Requests server to move
 * axis to a position relative to the current position (units depend on
 * controller but most often it's millimeters).  ESP_MOTION_DONE_STATUS will be
 * sent from the server to the client when it's done moving followed by a
 * ESP_POSITION message reporting the current position for that axis.
 */
#define ESP_MOVE_REL		(FIRST_ESP_MESSAGE+9)

/* 
 * Message(s): ESP_GET_POWER_STATUS
 * Direction: Client->Server
 * Data:  NULL
 * Description:  Sends request for the power status of all of the axes.
 * Triggers server to send the power status back to the all the clients in
 * message ESP_POWER_STATUS.
 */
#define ESP_GET_POWER_STATUS	(FIRST_ESP_MESSAGE+10)

/* 
 * Message(s): ESP_POWER_STATUS
 * Direction: Server->Client
 * Data:  char
 * Description:  Power status for each stage, which are packed into an 8-bit
 * char.  The first stage is the LSB.  1=ON and 0=OFF.  Look at the gtkclient
 * to see how to unpack the data.
 */
#define ESP_POWER_STATUS	(FIRST_ESP_MESSAGE+11)

/* 
 * Message(s): ESP_POSITION
 * Direction: Server->Client
 * Data:  struct AXIS
 * Description:  Contains the actual position (struct member 'real_position'
 * and beam position (struct member 'beam') for axis 'num'.  The beam positions
 * are 0 (OUT), 1-7, and -1 (none of the above).
 */
#define ESP_POSITION		(FIRST_ESP_MESSAGE+12)

/* 
 * Message(s): ESP_GET_POSITION
 * Direction: Server->Client
 * Data:  struct AXIS
 * Description:  Sends request for the position and beam positions of the "num"
 * axis.  Triggers server to send the positions and beam position back to the
 * all the clients in message ESP_POSITION.
 */
#define ESP_GET_POSITION	(FIRST_ESP_MESSAGE+13)

/* 
 * Message(s): ESP_MOTION_DONE_STATUS
 * Direction: Server->Client
 * Data:  char
 * Description:  Motion done status for each stage, which are packed into an
 * 8-bit char.  The first stage is the LSB.  1=MOVING and 0=DONE MOVING.  Look
 * at the gtkclient to see how to unpack the data.
 */
#define ESP_MOTION_DONE_STATUS	(FIRST_ESP_MESSAGE+14)

/* 
 * Message(s): ESP_GET_MOTION_DONE_STATUS
 * Direction: Client->Server
 * Data:  NULL
 * Description:  Sends request for the motion done status of all of the axes.
 * Triggers server to send the motion done status back to the all the clients
 * in message ESP_MOTION_DONE_STATUS.
 */
#define ESP_GET_MOTION_DONE_STATUS	(FIRST_ESP_MESSAGE+15)

/* 
 * Message(s): ESP_SET_VELOCITY
 * Direction: Client->Server
 * Data:  struct AXIS
 * Description:  Set the "num" to the axis number and "velocity" to the desired
 * velocity.  Requests server to set the velocity and max velocity of an axis
 * to the same value.
 */
#define ESP_SET_VELOCITY	(FIRST_ESP_MESSAGE+16)

/* 
 * Message(s): ESP_SET_ACCELERATION
 * Direction: Client->Server
 * Data:  struct AXIS
 * Description:  Set the "num" to the axis number and "acceleration" to the
 * desired acceleration.  Requests server to set the acceleration and max
 * acceleration of an axis to the same value.
 */
#define ESP_SET_ACCELERATION	(FIRST_ESP_MESSAGE+17)

/* 
 * Message(s): ESP_GET_MINMAX
 * Direction: Client->Server
 * Data: struct AXIS
 * Description:  Set the "num" to the axis number.  Requests that the server
 * send the min/max travels for the axis.  The server will respond with the
 * message ESP_MINMAX.
 */
#define ESP_GET_MINMAX		(FIRST_ESP_MESSAGE+18)

/* 
 * Message(s): ESP_MINMAX
 * Direction: Server->Client
 * Data: struct AXIS
 * Description:  Min/max travel limits (struct members "min" and "max") for
 * stage "num".
 */
#define ESP_MINMAX		(FIRST_ESP_MESSAGE+19)

/* 
 * Message(s): ESP_GET_VELOCITY
 * Direction: Client->Server
 * Data: struct AXIS
 * Description:  Set the "num" to the axis number.  Requests that the server
 * send the velocity for the axis.  The server will respond with the message
 * ESP_VELOCITY.
 */
#define ESP_GET_VELOCITY	(FIRST_ESP_MESSAGE+20)

/* 
 * Message(s): ESP_GET_ACCELERATION
 * Direction: Client->Server
 * Data: struct AXIS
 * Description:  Set the "num" to the axis number.  Requests that the server
 * send the acceleration for the axis.  The server will respond with the
 * message ESP_ACCELERATION.
 */
#define ESP_GET_ACCELERATION	(FIRST_ESP_MESSAGE+21)

/* 
 * Message(s): ESP_VELOCITY
 * Direction: Server->Client
 * Data: struct AXIS
 * Description:  Velocity (struct member "velocity") for stage "num".
 */
#define ESP_VELOCITY		(FIRST_ESP_MESSAGE+22)

/* 
 * Message(s): ESP_ACCELERATION
 * Direction: Server->Client
 * Data: struct AXIS
 * Description:  Acceleration (struct member "acceleration") for stage "num".
 */
#define ESP_ACCELERATION	(FIRST_ESP_MESSAGE+23)

/* 
 * Message(s): ESP_SET_BEAM
 * Direction: Client->Server
 * Data:  struct AXIS
 * Description:  Set the "num" to the axis number and "beam" to the
 * desired beam number.  Requests server to move the axis to a given beam.
 * Beam positions are set in a configuration file read by the server.
 */
#define ESP_SET_BEAM		(FIRST_ESP_MESSAGE+24)


#define LAST_ESP_MESSAGE	ESP_SET_BEAM

/************************************************************************/
/* METLAS messages 700-799						*/
/************************************************************************/

#define FIRST_METLAS_MESSAGE	700

/* 
 * The ON OFF message have one byte of data to say which port. 
 * These messages only sent from client to server.
 */

#define METLAS_ON		(FIRST_METLAS_MESSAGE+0)
#define METLAS_OFF		(FIRST_METLAS_MESSAGE+1)

/* The GET_POWER goes from client to server and has no data. */

#define METLAS_GET_POWER	(FIRST_METLAS_MESSAGE+2)

/* The GET_DIODE command asks if the diode is on or off */

#define METLAS_GET_DIODE	(FIRST_METLAS_MESSAGE+3)

/* The METLAS_FLUSH_SERIAL command askes server to clear the serial buffer */

#define METLAS_FLUSH_SERIAL	(FIRST_METLAS_MESSAGE+4)

/* POWER goes from server to client and has the amount of laser power in it */

#define METLAS_POWER		(FIRST_METLAS_MESSAGE+5)

/* DIODE goes from server to client and sends the state of the diode */

#define METLAS_DIODE		(FIRST_METLAS_MESSAGE+6)

#define LAST_METLAS_MESSAGE	METLAS_DIODE

/************************************************************************/
/* METROL messages 800-899						*/
/************************************************************************/

#define FIRST_METROL_MESSAGE	800

/* Client->Server request to open metroloy port, no data */

#define METROL_MESSAGE_OPEN_PORT	(FIRST_METROL_MESSAGE+0)

/* 
 * Client->Server requests a display to start data is display name 
 * Server->Client tells client we are displaying thing, no data.
 */

#define METROL_MESSAGE_START_DISPLAY	(FIRST_METROL_MESSAGE+1)

/*
 * Server->Client tells client we have stopped display, no data
 */ 

#define METROL_MESSAGE_STOP_DISPLAY	(FIRST_METROL_MESSAGE+2)

/* Client->Server/Server->Client, Channels to display. Data of type */

struct s_metrol_channels {
				int	chan1;
				int	chan2;
			};

#define METROL_MESSAGE_SELECT_CHANNELS		(FIRST_METROL_MESSAGE+3)

/* Client->Server Reuest for message of type METROL_MESSAGE_SELECT_CHANNELS */

#define METROL_MESSAGE_WHICH_CHANNELS		(FIRST_METROL_MESSAGE+4)

/* Client->Server, request to change scale. Int Data one of the signals below */

#define METROL_MESSAGE_CHANGE_SCALE		(FIRST_METROL_MESSAGE+5)

/* Server->Client, current volts and time per cm, data of struct below */

struct s_metrol_scale {
				float volts_per_div1;
				float volts_per_div2;
				float seconds_per_div;
			};

#define METROL_MESSAGE_SCALE		(FIRST_METROL_MESSAGE+6)

/* 
 * Client->Server requests a display to start data is display name 
 * Data following structure.
 * Server->Client tells client we are displaying thing, no data.
 */

struct s_metrol_mult {
			int active[12];
			char display[256];
		     };

#define METROL_MESSAGE_START_MULT	(FIRST_METROL_MESSAGE+7)

/*
 * Server->Client tells client we have stopped display, no data
 */ 

#define METROL_MESSAGE_STOP_MULT	(FIRST_METROL_MESSAGE+8)

#define LAST_METROL_MESSAGE			METROL_MESSAGE_SCALE

/************************************************************************/
/* signals, mostly used for within functions.				*/
/************************************************************************/

#define METROL_SIGNAL_STOP		1
#define METROL_SIGNAL_VUP		2
#define METROL_SIGNAL_VDN		3
#define METROL_SIGNAL_HUP		4
#define METROL_SIGNAL_HDN		5
#define METROL_SIGNAL_TOGGLE_SHOW_PS	6
#define METROL_SIGNAL_CLEAR_FAULT	7
#define METROL_SIGNAL_VUP_1		8
#define METROL_SIGNAL_VDN_1		9
#define METROL_SIGNAL_VUP_2		10
#define METROL_SIGNAL_VDN_2		11

/************************************************************************/
/* PICO messages 900-999						*/
/************************************************************************/

#define FIRST_PICO_MESSAGE	900

/*
 * The first message is one to get the names and types of each mirror 
 * THis message goes from client to server and should initiate a 
 * PICO_MIRROR_LIST message in return.
 */

#define PICO_GET_MIRROR_LIST	(FIRST_PICO_MESSAGE+0)

/*
 * This is the response message form the server for the above message.
 * It's should contain a mirror_list structure.
 */

#define MAX_MIRRORS	60

typedef struct smirror {

	int  axis_one;
	int  axis_two;
	int  pulses;
	char type;	/* COUPLED, UNCOUPLED or SINGLE */
	char name[81];

	} smirror;

struct SMIRROR_LIST {

	smirror mirrors[MAX_MIRRORS];
	int	n;		/* How many mirrors are there? */
	char	name[81];	/* Name of this server */

	};

#define PICO_MIRROR_LIST	(FIRST_PICO_MESSAGE+1)

/*
 * This message goes from CLIENT to SERVER and is a request to move a mirror
 * in a certain direction by a certain amount.
 * It should contain a morror_move structure that holds the relevant data.
 */

#define UP		0
#define DOWN		1
#define LEFT		2
#define RIGHT		3
#define FORWARD		4
#define BACKWARD	5
#define	COUPLED		6
#define UNCOUPLED	7
#define SINGLE		8

typedef struct smirror_move {

	int	mirror;		/* Which mirror? */
	int	direction;	/* Which way ? */
	int	pulses;		/* How far? */

	} mirror_move;

#define PICO_MOVE_MIRROR	(FIRST_PICO_MESSAGE+2)

/*
 * This message goes from CLIENT to SERVER and asks it to re-initialize the
 * hardware. The ID of the device is sent back.
 */

#define PICO_INIT		(FIRST_PICO_MESSAGE+3)

#define LAST_PICO_MESSAGE	PICO_INIT

/************************************************************************/
/* RPC messages 1000-1099						*/
/************************************************************************/

#define FIRST_RPC_MESSAGE	1000

/* 
 * The ON OFF message have one byte of data to say which port. 
 * These messages only sent from client to server.
 */

#define RPC_ON		(FIRST_RPC_MESSAGE+0)
#define RPC_OFF		(FIRST_RPC_MESSAGE+1)

/* The GET_STATUS goes from client to server and has no data. */

#define RPC_GET_STATUS	(FIRST_RPC_MESSAGE+2)

/* The STATUS goes from server to client and has one byte as data . */

#define RPC_STATUS	(FIRST_RPC_MESSAGE+3)

/* GET_NAMES goes from client to server and has no data */

#define RPC_GET_NAMES	(FIRST_RPC_MESSAGE+4)

/* goes from server to client and has the names as a set of strings as data */

#define RPC_NAMES	(FIRST_RPC_MESSAGE+5)

#define LAST_RPC_MESSAGE	RPC_NAMES

/************************************************************************/
/* SHUTTER messages 1100-1199						*/
/************************************************************************/

#define FIRST_SHUTTER_MESSAGE	1100

/* Open a shutter, argument single byte */

#define OPEN_SHUTTER		(FIRST_SHUTTER_MESSAGE+0)

/* Close a shutter, argument single byte */

#define CLOSE_SHUTTER		(FIRST_SHUTTER_MESSAGE+1)

/* Request or send shutter status, data as structure */

struct sshutter_status {
	unsigned char	shutter;
	unsigned char	status;
		       };
#define STATUS_SHUTTER		(FIRST_SHUTTER_MESSAGE+2)

/* Request or send configuration, data as structure */

#define MAX_CONFIGS	64
#ifndef NUMBER_SHUTTERS
#define NUMBER_SHUTTERS 16
#endif

extern struct sshutter_conf {
	int  number_confs;
	char name[81];
	char shutter_names[NUMBER_SHUTTERS][81];
	char conf_names[MAX_CONFIGS][81];
	unsigned char conf[MAX_CONFIGS][NUMBER_SHUTTERS];
		     } shutter_conf;

#define SHUTTER_CONF		(FIRST_SHUTTER_MESSAGE+3)

#define LAST_SHUTTER_MESSAGE	SHUTTER_CONF

/************************************************************************/
/* TELESCOPE messages 1200-1299						*/
/************************************************************************/

#define FIRST_TELESCOPE_MESSAGE	1200

/*
 * Client->Server, request a display socket, data in structure s_socket_name
 */

#define TELESCOPE_OPEN_DISPLAY_SOCKET	(FIRST_TELESCOPE_MESSAGE+0)

/* 
 * Client->Server, to put acqusition in/out or change to finder.
 * ACQ_IN takes data as a NULL terminated string. NO data for the others.
 */

#define TELESCOPE_ACQ_IN		(FIRST_TELESCOPE_MESSAGE+1)
#define TELESCOPE_ACQ_OUT		(FIRST_TELESCOPE_MESSAGE+2)
#define TELESCOPE_FINDER_IN		(FIRST_TELESCOPE_MESSAGE+3)
#define TELESCOPE_SPY1_IN		(FIRST_TELESCOPE_MESSAGE+4)
#define TELESCOPE_SPY2_IN		(FIRST_TELESCOPE_MESSAGE+5)
#define TELESCOPE_M10ALIGN_IN		(FIRST_TELESCOPE_MESSAGE+6)
#define TELESCOPE_DUST_IN		(FIRST_TELESCOPE_MESSAGE+7)
#define TELESCOPE_CH7_IN		(FIRST_TELESCOPE_MESSAGE+8)
#define TELESCOPE_CH8_IN		(FIRST_TELESCOPE_MESSAGE+9)

/* 
 * Client->Server, turn TV on and off and control bounding box.
 * TV_ON takes data as a NULL terminated string. NO data for the others.
 */

#define TELESCOPE_TV_ON			(FIRST_TELESCOPE_MESSAGE+10)
#define TELESCOPE_TV_OFF		(FIRST_TELESCOPE_MESSAGE+11)
#define TELESCOPE_TV_SBB		(FIRST_TELESCOPE_MESSAGE+12)
#define TELESCOPE_TV_UBB		(FIRST_TELESCOPE_MESSAGE+13)

/*
 * Messages for adjusting Brightness/contrast of TV. All Client-Server.
 * ALl of these induce the Server to send a TELESCOPE_BRIGHTNESS_CONTRAST
 * message with current brightness and contrast information.
 * Client->Server TELESCOPE_BRIGHTNESS_CONTRAST induces server to
 * just send brightness/contrast data if there is no data. If there is
 * data it is used to actually set the brightness and contrast.
 */

#define TELESCOPE_TV_BRIGHTNESS_UP	(FIRST_TELESCOPE_MESSAGE+14)
#define TELESCOPE_TV_BRIGHTNESS_DOWN	(FIRST_TELESCOPE_MESSAGE+15)
#define TELESCOPE_TV_CONTRAST_UP	(FIRST_TELESCOPE_MESSAGE+16)
#define TELESCOPE_TV_CONTRAST_DOWN	(FIRST_TELESCOPE_MESSAGE+17)

struct s_brightness_contrast {
	int brightness;
	int contrast;
			     };

#define TELESCOPE_TV_BRIGHTNESS_CONTRAST (FIRST_TELESCOPE_MESSAGE+18)

/*
 * Messages for controling the tracking origin. First two are both
 * Client->Server with no data.
 */

#define TELESCOPE_SET_ORIGIN		 (FIRST_TELESCOPE_MESSAGE+19)
#define TELESCOPE_CENTER_ORIGIN		 (FIRST_TELESCOPE_MESSAGE+20)

/* 
 * Next one is Client->Server requesting change in X or Y axis origin, data
 * is in form of an integer that defines the size of the change.
 */

#define TELESCOPE_X_ORIGIN_CHANGE	(FIRST_TELESCOPE_MESSAGE+21)
#define TELESCOPE_Y_ORIGIN_CHANGE	(FIRST_TELESCOPE_MESSAGE+22)

/*
 * Client->Server for turing tracking on and off. No data.
 */

#define TELESCOPE_TRACK_ON		(FIRST_TELESCOPE_MESSAGE+23)
#define TELESCOPE_TRACK_OFF		(FIRST_TELESCOPE_MESSAGE+24)

/* 
 * Client->Server
 * Messages for turning data receive on, off and for clearing lost count.
 */

#define TELESCOPE_REC_ON		(FIRST_TELESCOPE_MESSAGE+25)
#define TELESCOPE_REC_OFF		(FIRST_TELESCOPE_MESSAGE+26)

/* Client->Server set tip/tilt position to zero. No data */

#define TELESCOPE_TT_ZERO		(FIRST_TELESCOPE_MESSAGE+27)

/* Client->Server Set servo type, data one unsigned character */

#ifndef NO_SERVO
#define NO_SERVO                0 /* NOTE ALSO USED AS BITS!!! */
#define WOBBLER_SERVO           1
#define ACQUISITION_SERVO       2
#define BOTH_SERVO              3
#define FINDER_SERVO            4
#endif

#define TELESCOPE_SERVO			(FIRST_TELESCOPE_MESSAGE+28)

/* Client->Server run grab command. No data. */

#define TELESCOPE_GRAB			(FIRST_TELESCOPE_MESSAGE+29)

/* Client->Server Dis/En-able telescope motion, no data */

#define TELESCOPE_DISABLE		(FIRST_TELESCOPE_MESSAGE+30)
#define TELESCOPE_ENABLE		(FIRST_TELESCOPE_MESSAGE+31)

/* Client->Server time sync commands, no data */

#define TELESCOPE_TSYNC			(FIRST_TELESCOPE_MESSAGE+32)
#define TELESCOPE_TDATE			(FIRST_TELESCOPE_MESSAGE+33)

/* Client->Server Stow and xoffzero, no data */

#define TELESCOPE_STOW			(FIRST_TELESCOPE_MESSAGE+34)
#define TELESCOPE_XOFFZERO		(FIRST_TELESCOPE_MESSAGE+35)

/* Client-> Autodome control, data single char ON and OFF */

#ifndef ON
#define ON 1
#define OFF 0
#endif

#define TELESCOPE_AUTODOME		(FIRST_TELESCOPE_MESSAGE+36)

/* Client->Server Send dome to home, no data */

#define TELESCOPE_HOMEDOME		(FIRST_TELESCOPE_MESSAGE+37)

/* Client->Server Mirror cover open/close, data single char OPEN/CLOSE */

#ifndef OPEN_COVER
#define OPEN_COVER 0
#define CLOSE_COVER 1
#endif 

#define TELESCOPE_PRIMARY_COVER		(FIRST_TELESCOPE_MESSAGE+38)
#define TELESCOPE_M3_COVER		(FIRST_TELESCOPE_MESSAGE+39)
#define TELESCOPE_FINDER_COVER		(FIRST_TELESCOPE_MESSAGE+40)
#define TELESCOPE_M5_COVER		(FIRST_TELESCOPE_MESSAGE+41)
#define TELESCOPE_DOME_SLIT		(FIRST_TELESCOPE_MESSAGE+42)

/*
 * Client->Server Set EL/AZ for telescope, data in structure below 
 */

struct s_elaz {
		double	el; /* In deciaml degrees! */
		double  az; /* In deciaml degrees! */
	      };

#define TELESCOPE_ELAZ	 	  	(FIRST_TELESCOPE_MESSAGE+43)

/* Client->Server Set RA/DEC for telescope, data in structure below */

struct s_radec {
		double	ra; /* In radians! */
		double  dec; /* In radians! */
	      };

#define TELESCOPE_RADEC	 	  	(FIRST_TELESCOPE_MESSAGE+44)

/* Client->Server, Set HR# of object to move to, data an int */

#define TELESCOPE_HR	 	  	(FIRST_TELESCOPE_MESSAGE+45)

/* Client->Server Send scope to next position, no data */

#define TELESCOPE_NEXT			(FIRST_TELESCOPE_MESSAGE+46)

/* Client->Server, change paddle setting, argument structure */

#ifndef PAD_STOP
#define PAD_STOP                0
#define PAD_UP                  1
#define PAD_DOWN                2
#define PAD_EAST                3
#define PAD_WEST                4
#endif

struct s_paddle {
		float rate;
		char direction;
		};

#define TELESCOPE_PADDLE		(FIRST_TELESCOPE_MESSAGE+47)

/* Client->Server, request to flush com port, no data */

#define TELESCOPE_FLUSH_COM_PORT	(FIRST_TELESCOPE_MESSAGE+48)

/*
 * Client->Server, request receive rate, no data
 * Server->Client, return receive rate, data int
 */

#define TELESCOPE_RECEIVE_RATE		(FIRST_TELESCOPE_MESSAGE+49)

/* Client->Server turns TCS rotation stuff ON or OFF, no data. */

#define TELESCOPE_START_TCS		(FIRST_TELESCOPE_MESSAGE+50)
#define TELESCOPE_STOP_TCS		(FIRST_TELESCOPE_MESSAGE+51)

/* 
 * Client->Server, request current telescope delta, no data.
 * Server->Client return current telescope delta, data float.
 */

#define TELESCOPE_DELTA			(FIRST_TELESCOPE_MESSAGE+52)

/*
 * Client->telescope, request check for star, no data.
 * Server->Client, response. Data char, TRUE if star is there.
 */

#define TELESCOPE_CHECK_FOR_STAR	(FIRST_TELESCOPE_MESSAGE+53)

/* 
 * Client-Serevr, request acqusition tracking error, no data
 * Server->Client, return acqusition tracking error, data float
 */

#define TELESCOPE_CHECK_FOR_LOCK	(FIRST_TELESCOPE_MESSAGE+54)

/* Client->Server request initialization of TCS, no data */

#define TELESCOPE_TCS_INIT		(FIRST_TELESCOPE_MESSAGE+55)

/* Client->Server Request TEMA timeout clear. */

#define TELESCOPE_TEMA_CLEAR_TIMEOUT	(FIRST_TELESCOPE_MESSAGE+56)

/* Messages for controlling TV zoom, no data, client->server only. */

#define TELESCOPE_TV_ZOOM_IN		(FIRST_TELESCOPE_MESSAGE+57)
#define TELESCOPE_TV_ZOOM_OUT		(FIRST_TELESCOPE_MESSAGE+58)
#define TELESCOPE_TV_ZOOM_LEFT		(FIRST_TELESCOPE_MESSAGE+59)
#define TELESCOPE_TV_ZOOM_RIGHT		(FIRST_TELESCOPE_MESSAGE+60)
#define TELESCOPE_TV_ZOOM_UP		(FIRST_TELESCOPE_MESSAGE+61)
#define TELESCOPE_TV_ZOOM_DOWN		(FIRST_TELESCOPE_MESSAGE+62)

/* CLient->Server, a request for message type TELESCOPE_SET_TV_MEAN_N */

#define TELESCOPE_GET_TV_MEAN_N		(FIRST_TELESCOPE_MESSAGE+63)

/* Client->Server, or Server->CLient set tv mean n, data an int */

#define TELESCOPE_SET_TV_MEAN_N		(FIRST_TELESCOPE_MESSAGE+64)

/* Client->Server for going into TEMA local mode */

#define TELESCOPE_TEMA_LOCAL		(FIRST_TELESCOPE_MESSAGE+65)

/* Client->Server for collection of pointing model data */

#define TELESCOPE_OPEN_POINT_DATA	(FIRST_TELESCOPE_MESSAGE+66)
#define TELESCOPE_CLOSE_POINT_DATA	(FIRST_TELESCOPE_MESSAGE+67)
#define TELESCOPE_COLLECT_POINT_DATA	(FIRST_TELESCOPE_MESSAGE+68)

/* Client->Server FINDSTAR for telescope, data in structure below */

#define TELESCOPE_FINDSTAR		(FIRST_TELESCOPE_MESSAGE+69)

struct s_findstar {
		float	el; /* In deciaml degrees! */
		float  az; /* In deciaml degrees! */
		float   mag; /* Minimum magnitude */
	      };

/* 
 * Client->Server request telescope position, no data 
 * Server->Client return of scope position, data as struct below.
 */

#define TELESCOPE_POSITION		(FIRST_TELESCOPE_MESSAGE+70)

struct s_scope_pos {
		double	el; /* In decimal degrees! */
		double  az; /* In decimal degrees! */
		double	ra; /* In radians! */
		double  dec; /* In radians! */
		char	star_name[32];
	      };

/* Client->Server, request aquire function start going */

#define TELESCOPE_ACQUIRE		(FIRST_TELESCOPE_MESSAGE+71)

/* Client->Server, controls heaters at TEMA, data char TRUE for ON. */

#define TELESCOPE_DEHUMID_HEAT		(FIRST_TELESCOPE_MESSAGE+72)

/* 
 * CLient Server, request status of covers. no data.
 * Server->Client, returns result of status, data as struct below.
 */

#define TELESCOPE_TEMA_STATUS		(FIRST_TELESCOPE_MESSAGE+73)

struct s_tema_status {
			int	primary;
			int	m3;
			int 	finder;
			int 	m5;
			int	acq;
		     };

/* CLIent -> server, start the get_Star sequence. no data */

#define TELESCOPE_GET_STAR		(FIRST_TELESCOPE_MESSAGE+74)

/*
 * Server->Client: Send TV image. Image is compressed and is the
 * small 8 bit image as defined in the telescope code. Data is this image
 * and the length will depend on the compression. The uncompressed images
 * should always be 320x240.
 */

#define TELESCOPE_DISPLAY_TV		(FIRST_TELESCOPE_MESSAGE+75)

/* Server->Client: Stop TV display. No data. */

#define TELESCOPE_STOP_TV		(FIRST_TELESCOPE_MESSAGE+76)

/*
 * Server->Client: Send TV picture window name. Data string with no termination.
 */

#define TELESCOPE_DISPLAY_NAME		(FIRST_TELESCOPE_MESSAGE+77)

/* Client->Server Request a dust measurement. No data for now. */

#define TELESCOPE_DUST			(FIRST_TELESCOPE_MESSAGE+78)

/* Client->Server Zero or request a bias measurement. No data for now. */

#define TELESCOPE_MKBIAS		(FIRST_TELESCOPE_MESSAGE+79)
#define TELESCOPE_ZBIAS			(FIRST_TELESCOPE_MESSAGE+80)

/* Server->Client status message for displaying in scopetab. Data structure */

#define SCOPE_ACQ_OUT 		0
#define SCOPE_ACQ_IN 		1
#define SCOPE_ACQ_MOVING 	2
#define SCOPE_ACQ_IMPOSSIBLE 	3

struct s_scope_status {
	double el;
	double az;
	double ra;
	double dec;
	int    acq;
	char	star_name[36];
		};

#define TELESCOPE_STATUS		(FIRST_TELESCOPE_MESSAGE+81)
	
#define LAST_TELESCOPE_MESSAGE		TELESCOPE_STATUS

/************************************************************************/
/* And now we define the signals that TELESCOPE knows about.		*/
/************************************************************************/

#define TELESCOPE_SIGNAL_EXIT			1
#define TELESCOPE_SIGNAL_GO			2
#define TELESCOPE_SIGNAL_UP			3
#define TELESCOPE_SIGNAL_DOWN			4
#define TELESCOPE_SIGNAL_LEFT			5
#define TELESCOPE_SIGNAL_RIGHT			6
#define TELESCOPE_SIGNAL_BIGGER			7
#define TELESCOPE_SIGNAL_SMALLER		8
#define TELESCOPE_SIGNAL_READY_FOR_DISPLAY	9

/************************************************************************/
/* CENTRAL_SCRUTINIZER messages 1300-1399				*/
/************************************************************************/

#define FIRST_CENTRAL_SCRUTINIZER_MESSAGE	1300

/* 
 * Server->Client A message for the status window and log file. Normally
 * only meant for the central_scrutinizer client. All other clients should 
 * simply ignore this message. Data the string itself, not including
 * terminating NULL.
 */

#define CENTRAL_SCRUTINIZER_LOG		(FIRST_CENTRAL_SCRUTINIZER_MESSAGE+0)

/*
 * Server-> Client A message to stop the job queue. Only used by the
 * Central Scrutinizer or Grand Wazoo. All other clients should ignore this.
 */

#define CENTRAL_SCRUTINIZER_STOP_JOB_QUEUE (FIRST_CENTRAL_SCRUTINIZER_MESSAGE+1)

#define LAST_CENTRAL_SCRUTINIZER_MESSAGE	CENTRAL_SCRUTINIZER_LOG

/************************************************************************/
/* SOCKET_MANAGER messages 1400-1499					*/
/************************************************************************/

#define FIRST_SOCKET_MANAGER			1400

/*
 * Client->Server, Request for a process's socket info. Data is a string
 * representing the name of the process you are interested in.
 */

#define SOCKET_MANAGER_GET_PROCESS		(FIRST_SOCKET_MANAGER+0)

/* 
 * Client->Server Request to register a process's socket numbers
 * data in form of structure.
 * Server->Client Response to SOCKET_MANAGER_GET_PROCESS
 */

struct s_process_sockets {
			int	pid;		/* PID of server process */
			int	commands;	/* Command port */
			int	data;		/* Data port */
			int	messages;	/* Message port */
			char	restart_command[256]; /* Guess */
			char	name[80];	/* Process name */
			char	machine[80];	/* Computer name */
		 };

#define SOCKET_MANAGER_SET_PROCESS		(FIRST_SOCKET_MANAGER+1)

/*
 * Client->Server, Request to remove process from the list. Data string
 * representing the name of the process you are interested in.
 * Server should reposnd with a SOCKET_MANAGER_SET_PROCESS message
 * with all socket numbers set to -1.
 */

#define SOCKET_MANAGER_REMOVE_PROCESS		(FIRST_SOCKET_MANAGER+2)

/*
 * Client->Server, Request a list of all process names. No data.
 * Server->Client, Returns list. Data is one long NULL terminated string
 *                 containing a list of '\n' separated names.
 */

#define SOCKET_MANAGER_LIST_NAMES		(FIRST_SOCKET_MANAGER+3)

/* Server->Client, lets client know no such process exists. No data */

#define SOCKET_MANAGER_NO_SUCH_PROCESS		(FIRST_SOCKET_MANAGER+4)

#define LAST_SOCKET_MANAGER			(SOCKET_MANAGER_NO_SUCH_PROCESS)

/************************************************************************/
/* POP_SERVER messages 1500-1599					*/
/************************************************************************/

#define FIRST_POP_MESSAGE			1500

/* Structure used for most messages */

struct POP{
                int id;
                char name[81];
                char status;
                unsigned char position;
                char status_limit;
                char status_motor;
                char status_led;
                char status_timeout;
                char status_local;
           };

/* 
 * Message(s): POP_[IN,OUT,OFF,LED_[RED,GREEN,YELLOW,OFF]]
 * Direction: Client->Server
 * Data:  struct POP
 * Description:  Set the "id" member to the pop number. Requests server to do
 * as the command suggests.  Server responds by sending POP_STATUS messages to
 * all the clients until the task has been completed.
 */
#define POP_IN					FIRST_POP_MESSAGE+0
#define POP_OUT					FIRST_POP_MESSAGE+1
#define POP_OFF					FIRST_POP_MESSAGE+2
#define POP_LED_RED				FIRST_POP_MESSAGE+3
#define POP_LED_GREEN				FIRST_POP_MESSAGE+4
#define POP_LED_YELLOW				FIRST_POP_MESSAGE+5
#define POP_LED_OFF				FIRST_POP_MESSAGE+6

/* 
 * Message(s): POP_SEND_STATUS
 * Direction: Client->Server
 * Data:  struct POP
 * Description:  Set the "id" member to the pop number. Requests server to get
 * the POP status and send it via the POP_STATUS message to all the clients.
 */
#define POP_SEND_STATUS				FIRST_POP_MESSAGE+7


/* 
 * Message(s): POP_STATUS
 * Direction: Server->Client
 * Data:  struct POP
 * Description:  The "id" member is set to the pop number.  
 * The status bit has the form (LSB on the right):
 * Status: 0  0  1  1  0  0  1  0
 *         |  |  |__|  |__|  |__|
 *      Manual|   LED  Motor R  F
 *         Timeout         Limits 
 *  Other status members are status_limit, status_motor, status_led,
 *  status_timeout, status_local which are explained in pop_cliserv.h
 */
#define POP_STATUS				FIRST_POP_MESSAGE+8

/* that's all folks */
#define LAST_POP_MESSAGE			POP_STATUS

/************************************************************************/
/* WEATHER_SERVER messages 1600-1699					*/
/************************************************************************/

#define FIRST_WEATHER_MESSAGE			1600

/* Server to client, for pushing the current conditions */

#define WEATHER_NOW				(FIRST_WEATHER_MESSAGE+0)

struct s_weather_data {
			float hour_ut;		/* Time in fractional hours */
			float wind_speed;
			float wind_dir;
			float gust;
			float gust_dir;
			float outdoor_temp;
			float indoor_temp;
			float outdoor_humid;
			float indoor_humid;
			float dew_point;
			float wind_speed_avg;
			float barometric_pressure;
		      };

/*
 * Client-Server, request current data file name.
 * Server->Client, response, data is the filename not 
 * including terminating null.
 */

#define WEATHER_FILE_NAME			(FIRST_WEATHER_MESSAGE+1)

#define LAST_WEATHER_MESSAGE			WEATHER_FILE_NAME

/************************************************************************/
/* THORLABS_FW102 messages 1700-1799					*/
/************************************************************************/

#define FIRST_THORLABS_FW102_MESSAGE			1700

/*
 * Client->Server Request filter names list no data
 * Server->Client Send name list, data in structure below
 */

#define THORLABS_FW102_NAMES			(FIRST_THORLABS_FW102_MESSAGE+0)

struct s_thorlabs_fw102_names {
		char	unit_name[81];
		char	filter_names[6][81];
			      };

/*
 * Client->Server Request new filter, data char, new filter
 */

#define THORLABS_FW102_SET_FILTER		(FIRST_THORLABS_FW102_MESSAGE+1)

/*
 * Client->Server Request unit ID, no data
 */

#define THORLABS_FW102_IDN			(FIRST_THORLABS_FW102_MESSAGE+2)

/*
 * Client->Server Request current filter, no data
 * Server->Client Reply, data char=current filter
 */

#define THORLABS_FW102_CURRENT_FILTER		(FIRST_THORLABS_FW102_MESSAGE+3)

/* Client->Server Reuest a reopening of the serial port. */

#define THORLABS_FW102_REOPEN_PORT		(FIRST_THORLABS_FW102_MESSAGE+4)

#define LAST_THORLABS_FW102_MESSAGE		THORLABS_FW102_REOPEN_PORT

/************************************************************************/
/* FAST_CLOCK messages 1800-1899					*/
/************************************************************************/

#define FIRST_FAST_LOCK_MESSAGE			1800

/* Client->Server, clears lost ticks. No data. */

#define FAST_CLOCK_CLEAR_LOST_TICKS		(FIRST_FAST_LOCK_MESSAGE)

/* 
 * Client->Server, Initiate a fast clock calibration, 
 * data unsigned char number of seconds.
 */

#define FAST_CLOCK_CALIBRATE			(FIRST_FAST_LOCK_MESSAGE+1)

/*
 * Server->Client sending information about the calibration process.
 * Data unsigned char, number of seconds left.
 */

#define FAST_CLOCK_CALIBRATE_INFO		(FIRST_FAST_LOCK_MESSAGE+2)

/* Client->Server For setting the clock period. Data integer. */

#define FAST_CLOCK_SET_PERIOD			(FIRST_FAST_LOCK_MESSAGE+3)

/* Client->Server, request the curretn rate. No data */

#define FAST_CLOCK_GET_RATE			(FIRST_FAST_LOCK_MESSAGE+4)

/* 
 * Client->Server For setting the clock rate. Data integer.
 * Server->Client as a response to FAST_CLOCK_GET_RATE. Data integer.
 */

#define FAST_CLOCK_SET_RATE			(FIRST_FAST_LOCK_MESSAGE+5)

/* 
 * Client->Server, request the time. No data.
 * Server->Client, sending the time, Data fast_clock_time_struct.
 */

#ifndef __CLOCK__
#ifndef __CLOCK_CLISERV__
#define CHARA_TIME int
#endif
#endif
#ifndef FAST_CLOCK_TIME_STRUCT_DEFINED
typedef struct FAST_CLOCK_TIME_STRUCT
        {
                CHARA_TIME time_now;
                CHARA_TIME lost_ticks;
                                                                                
        } fast_clock_time_struct;
#define FAST_CLOCK_TIME_STRUCT_DEFINED
#endif

#define FAST_CLOCK_TIME                     	(FIRST_FAST_LOCK_MESSAGE+6)

/* Client->Server to force a sync of the fast clock to the chara clock */

#define FAST_CLOCK_SYNC                     	(FIRST_FAST_LOCK_MESSAGE+7)

#define LAST_FAST_CLOCK_MESSAGE			FAST_CLOCK_SYNC

/************************************************************************/
/* signals, mostly used for within functions.				*/
/************************************************************************/

#define FAST_CLOCK_SIGNAL_STOP		1

/************************************************************************/
/* TARGET messages 1900-1999                                            */
/************************************************************************/

#define FIRST_TARGET_MESSAGE                    1900

/* 
 * Message(s): TARGET_[UP,DOWN,LED_ON,LED_OFF,RESET,REMOTE]
 * Direction: Client->Server
 * Data:  struct TARGET
 * Description:  Set the "id" member to the target number. Requests server to
 * do as the command suggests.  Server responds by sending TARGET_STATUS
 * messages to all the clients until the task has been completed.
 */
#define TARGET_UP                               (FIRST_TARGET_MESSAGE)
#define TARGET_DOWN                             (FIRST_TARGET_MESSAGE+1)
#define TARGET_LED_ON                           (FIRST_TARGET_MESSAGE+2)
#define TARGET_LED_OFF                          (FIRST_TARGET_MESSAGE+3)
#define TARGET_RESET                            (FIRST_TARGET_MESSAGE+4)
#define TARGET_REMOTE                           (FIRST_TARGET_MESSAGE+5)


/* 
 * Message(s): TARGET_GET_STATUS
 * Direction: Client->Server
 * Data:  struct TARGET
 * Description:  Set the "id" member to the TARGET number. Requests server to
 * get the target status and send it via the TARGET_STATUS message to all the
 * clients.
 */
#define TARGET_GET_STATUS                       (FIRST_TARGET_MESSAGE+6)


/* 
 * Message(s): TARGET_STATUS
 * Direction: Server->Client
 * Data:  struct TARGET
 * Description:  The "id" member is set to the TARGET number.  The
 * target_status and led_status members are set.  See target_cliserv.h for
 * possible values.
 */
#define TARGET_STATUS                           (FIRST_TARGET_MESSAGE+7)

#define LAST_TARGET_MESSAGE                     (TARGET_STATUS)

/************************************************************************/
/* TIPTILT messages 2000-2099                                           */
/************************************************************************/

#define FIRST_TIPTILT_MESSAGE                    2000

/*
 * Client->Server for setting exposure time, data integer exposure time mS
 */

#define TIPTILT_EXPOSURE		(FIRST_TIPTILT_MESSAGE+0)

/*
 * Client->Server for setting things to FULL FRAME mode. No data
 */

#define TIPTILT_FULL_FRAME		(FIRST_TIPTILT_MESSAGE+1)

/*
 * Client->Server for setting things to QUAD FRAME mode. No data
 */

#define TIPTILT_QUAD_FRAME		(FIRST_TIPTILT_MESSAGE+2)

/* 
 * Client->Server request for current sub frame. No data.
 */

#define TIPTILT_GET_SUB_FRAME		(FIRST_TIPTILT_MESSAGE+3)

/*
 * Client->Server for setting things to FULL FRAME mode. Data is struct
 * Server->Client in answer to a TIPTILT_GET_SUB_FRAME. Data is struct
 */

struct s_tiptilt_sub {
			int x;
			int y;
			int dx;
			int dy;
		     };

#define TIPTILT_SUB_FRAME		(FIRST_TIPTILT_MESSAGE+4)

/* Client->Server select quadrant to use. Data int. */

#define TIPTILT_SET_QUADRANT		(FIRST_TIPTILT_MESSAGE+5)

/* Client->Server request display of a single frame, data */

struct s_tiptilt_display_frame {
			int  pixmult;
			char display_name[256];
			       };

#define TIPTILT_DISPLAY_FRAME		(FIRST_TIPTILT_MESSAGE+6)

/* Client->Server request to clear the X display */

#define TIPTILT_CLEAR_WINDOW		(FIRST_TIPTILT_MESSAGE+7)

/* Client->Server request Surface plot, data name of display */

#define TIPTILT_SURFACE_FRAME		(FIRST_TIPTILT_MESSAGE+8)

/* Client->Server request new bias calculation, data struct . */

struct s_tiptilt_bias {
			int num_frames;
			char save_bias;
		      };

#define TIPTILT_MAKE_BIAS		(FIRST_TIPTILT_MESSAGE+9)

/* Client->Server request zeroing of bias. No data. */

#define TIPTILT_ZERO_BIAS		(FIRST_TIPTILT_MESSAGE+10)

/* Client->Server request line plot, data os structure: */

struct s_tiptilt_line{
			int x;
			int y;
			char display_name[256];
		     };

#define TIPTILT_LINE			(FIRST_TIPTILT_MESSAGE+11)

/* Client->Server request movie, data structure */

struct s_movie {
			int type;
			char display[256];
		};

#define FRAME_MOVIE 0
#define SURF_MOVIE  1
#define LINE_MOVIE  2

#define TIPTILT_MOVIE			(FIRST_TIPTILT_MESSAGE+12)

/* Server->Client shows the current status of movie, data in structure */

struct s_movie_status {
			float	min;
			float	max;
			float	frame_rate;
			int	frame_number;
		      };

#define TIPTILT_MOVIE_STATUS		(FIRST_TIPTILT_MESSAGE+13)

/* Server->Clients Stops movie status windows. */

#define TIPTILT_MOVIE_STATUS_STOP     	(FIRST_TIPTILT_MESSAGE+14)

/* CLient->Server for controlling the display socket, data s_socket_name */

#define TIPTILT_OPEN_DISPLAY_SOCKET	(FIRST_TIPTILT_MESSAGE+15)

/*
 * Clinet->Server request number of channels. No data
 * Server->Client Actual number of channels, data integer.
 */

#define TIPTILT_NUMBER_CHANNELS		(FIRST_WOBBLE_MESSAGE+16)

/*
 * Client->Server request chanel names, no data
 */
                                                                                
#define TIPTILT_NAMES  			(FIRST_WOBBLE_MESSAGE+17)
                                                                                
/*
 * Server->Client list of names as per structure
 */
                                                                                
struct s_tiptilt_names {
		char names[NUM_SCOPES][81];
	       };

/*
 * Client->Server set scope names, data structre above.
 */

#define TIPTILT_SCOPE_NAMES		(FIRST_TIPTILT_MESSAGE+18)

/*
 * Client->server request status information on one channel.
 * Data is an integer specifying the chanel.
 */

#define TIPTILT_STATUS			(FIRST_TIPTILT_MESSAGE+19)

/*
 * Server->Client status information, data in following structure
 */

struct s_tiptilt_status {
			int	      det;   /* Which detector */
			int	      n_sum;  /* How many samples */
			unsigned char on;    /* Is the servo on? */
			unsigned char mod_x; /* TRUE or FALSE */
			unsigned char mod_y; /* TRUE or FALSE */
			unsigned char send_on; /* TRUE or FALSE */
		};

/*
 * Client->server, for turing on/off servo, modulation and signals
 * all have integer (chanel) for arguement
 */

#define TIPTILT_ON			(FIRST_TIPTILT_MESSAGE+20)
#define TIPTILT_OFF			(FIRST_TIPTILT_MESSAGE+21)
#define TIPTILT_MODX			(FIRST_TIPTILT_MESSAGE+22)
#define TIPTILT_MODY			(FIRST_TIPTILT_MESSAGE+23)
#define TIPTILT_SENDON			(FIRST_TIPTILT_MESSAGE+24)
#define TIPTILT_SENDOFF			(FIRST_TIPTILT_MESSAGE+25)

/*
 * Client->Server requests to start/stop interrupt callback routine.
 */

#define TIPTILT_START			(FIRST_TIPTILT_MESSAGE+26)
#define TIPTILT_STOP			(FIRST_TIPTILT_MESSAGE+27)

/*
 * Clinet->Server request to reinitialize the camera
 */

#define TIPTILT_INIT			(FIRST_TIPTILT_MESSAGE+28)

/* Client->Server request current faint limit, no data */

#define TIPTILT_GET_FAINT		(FIRST_TIPTILT_MESSAGE+29)

/*
 * CLient->Server Set faint limit
 * Server->Client response to TIPTILT_GET_FAINT, both data float
 */

#define TIPTILT_SET_FAINT               (FIRST_TIPTILT_MESSAGE+30)

/* 
 * Client->Server, request counts seen in system, no data.
 * Server->Client, return counts seen, data of form
 * float counts[NUM_SCOPES];
 */

#define TIPTILT_COUNTS			(FIRST_TIPTILT_MESSAGE+31)

/* Client->Server, requests logging of seeing, no data */

#define TIPTILT_LOG_SEEING		(FIRST_TIPTILT_MESSAGE+32)

/* Client->Server, open or close the log file, no data */

#define TIPTILT_OPEN_LOG		(FIRST_TIPTILT_MESSAGE+33)
#define TIPTILT_CLOSE_LOG		(FIRST_TIPTILT_MESSAGE+34)

/* 
 * Client->Server, request errors seen in system, no data.
 * Server->Client, return errors seen, data of form
 * float errors[NUM_SCOPES];
 */

#define TIPTILT_ERRORS			(FIRST_TIPTILT_MESSAGE+35)

/* Client->Server, set global picture minimum and maximum */

struct s_tiptilt_minmax {
		float min;
		float max;
			};

#define TIPTILT_MINMAX			(FIRST_TIPTILT_MESSAGE+36)

/*
 * CLient->Server turn signals on or off. No data
 */

#define TIPTILT_SIGON            	(FIRST_TIPTILT_MESSAGE+37)
#define TIPTILT_SIGOFF           	(FIRST_TIPTILT_MESSAGE+38)

/*
 * Client->Server, request seeing loig filename. NO data
 * Server->Client, Send seeing log filename, data filename including terminating
 * null.
 */

#define TIPTILT_LOG_FILENAME		(FIRST_WOBBLE_MESSAGE+39)

/* Client->Server Toggle showing pixel positions on display. No Data */

#define TIPTILT_TOGGLE_SHOW_POSITIONS	(FIRST_WOBBLE_MESSAGE+40)

/* Client->Server for controlling the display socket, data s_socket_name */

#define TIPTILT_CLOSE_DISPLAY_SOCKET	(FIRST_TIPTILT_MESSAGE+41)

/* Client->Server Request logging of tiptilt errors. Used only by irimage. */

#define TIPTILT_IRIMAGE_TIPTILT_LOG	(FIRST_TIPTILT_MESSAGE+42)
struct s_irimage_tiptilt_log
	{
		char	filename[256];
		CHARA_TIME start_time;
		CHARA_TIME stop_time;
	};

/* Client->Server: Make detector bias for single chanel. Data integer chanel */

#define TIPTILT_MAKE_DETECTOR_BIAS	(FIRST_TIPTILT_MESSAGE+43)

/* Client->Server: Zero detector bias for single chanel. Data integer chanel */

#define TIPTILT_ZERO_DETECTOR_BIAS	(FIRST_TIPTILT_MESSAGE+44)

/* Client-> Server: Set n_sum for one chanel. Data structure below */

#define TIPTILT_N_SUM			(FIRST_TIPTILT_MESSAGE+45)

struct s_tiptilt_n_sum
	{
		int det;
		int n_sum;
	};

/* Client -> Server: Set vasrriance period. Data Integer number of seconds */

#define TIPTILT_SET_VAR_PERIOD		(FIRST_TIPTILT_MESSAGE+46)

/* Server->Client: Send current display information. Data array of structure. */

#define TIPTILT_DISPLAY		        (FIRST_TIPTILT_MESSAGE+47)

struct s_tiptilt_display {
	float   rot_x,rot_y;
        float   Az,El;
                          };

/* Server->Client Sends seeing information. Data float seeing[NUM_SCOPES] */

#define TIPTILT_SEEING			(FIRST_TIPTILT_MESSAGE+48)

#define LAST_TIPTILT_MESSAGE            (TIPTILT_SEEING)

/************************************************************************/
/* And now we define the signals that TIPTILT knows about.		*/
/************************************************************************/

#define TIPTILT_SIGNAL_EXIT			1
#define TIPTILT_SIGNAL_READY_FOR_DISPLAY	2

/************************************************************************/
/* ASTROMOD messages 2100-2199                                          */
/*									*/
/* THese are generic messages that a number of servers should be able	*/
/* to deal with. Any server that needs to know which star we are	*/
/* looking at or where the star is or where the delay lines should be	*/
/* should know about these messages.					*/
/************************************************************************/

#define FIRST_ASTROMOD_MESSAGE                    2100

/* Client->Server, request for current configuration. No data */

#define ASTROMOD_CURRENT_CONFIG       	(FIRST_ASTROMOD_MESSAGE+0)

/*
 * Client-> Server forces server to re-read model parameters. Data in
 * structure define below. To force the use of default pops/beams set
 * any values to -1 in the appropriate array.
 * Server->Client, responce to ASTROMOD_CURRENT_CONFIG.
 */

#define ASTROMOD_INIT			(FIRST_ASTROMOD_MESSAGE+1)

typedef struct s_astromod_init{
		int beams[NUM_SCOPES];
		int pops[NUM_SCOPES];
		      } astromod_init_struct;

/*
 * Client->Server, request to see which star has been set. No data.
 * Response is an ASTROMOD_SET_STAR message. If no star is set, chara
 * number returned is <0.
 */

#define ASTROMOD_WHICH_STAR		(FIRST_ASTROMOD_MESSAGE+2)

/*
 * Client->Server. Set the star information. Data integer, 
 * CHARA number.
 * for star or one of the defines below.
 * Server->Client, responce to ASTROMOD_WHICH_STAR, Data integer CHARA number
 * for star, <0 for defines as listed.
 */

#define STAR		 1
#define NOSTAR		 0
#define	ZENITH		-1
#define INTERNAL 	-2
#define AUTOCOLL 	-3
#define ASTROMOD_SET_STAR		(FIRST_ASTROMOD_MESSAGE+3)

/* 
 * Client->Server, request to see which cart is set as reference, no data.
 * Response is an ASTROMOD_SET_REFERENCE_CART message.]
 */

#define ASTROMOD_WHICH_REFERENCE_CART   (FIRST_ASTROMOD_MESSAGE+4)

/*
 * Client->Server, Set reference cart, data an integer.
 * Server->Client, Response to ASTROMOD_WHICH_REFERENCE_CART.
 */

#define ASTROMOD_SET_REFERENCE_CART	(FIRST_ASTROMOD_MESSAGE+5)

/* Client->Server, set global boolian. Data an integer. */

#define ASTROMOD_SET_USE_ASTROMOD_LIB	(FIRST_ASTROMOD_MESSAGE+6)

/* 
 * Client->server request star information no data.
 * Server->Client send star information, data of type struct s_star
 */

#ifndef __CHARA__
struct  s_star {
                double ra;         /*  right ascension (rad) j          */
                double dec;        /*  declination (rad)                */
                double pm_ra;      /*  ra proper motion (rad/year)      */
                double pm_dec;     /*  dec proper motion (rad/yr)       */
                double parallax;   /*  parallax (arcseconds)            */
                double radvel;     /*  radial velocity (km/s)           */
                double vmag;       /*  V magnitude                      */
                double kmag;       /*  K magnitude                      */
                char name[32];     /*  star ID                          */
                char irc[32];      /*  IRC number of star               */
                char hr[32];       /*  HR number of star                */
                char hd[32];       /*  HD number of star                */
                char sao[32];      /*  SAO number of star               */
                char fk5[32];      /*  FK5 number of star CAT5          */
                char hip[32];      /*  HIP number of star CAT6          */
                char gj[32];       /*  GJ number of star CAT7           */
                char twomass[32];    /*  2 Mass number of star CAT8       */
                char rastr[32];    /*  ra display string                */
                char decstr[32];   /*  dec display string               */
                char spectype[32]; /*  spectral type                    */
                long long int chara_number;  /*  CHARA catalogue number.*/
                };
#endif

#define ASTROMOD_STAR_DATA		(FIRST_ASTROMOD_MESSAGE+7)

/*
 * Client->Server Request information on Alt/Az and so on. Data is s_time_now.
 * If year is zero it will use it's own local clock.
 * Units of UT 0->2PI = 0->24 hours
 * Wavelegth in microns.
 * Server->Client Replay with results. Data as s_uv. 
 * Results in radians and cycles per arcsecond.
 */

#define U 0
#define V 1

struct s_time_now {
		double ut;
		double lambda;
		int year;
		int month;
		int day;
		int dumby; /* To make it work for 32/64 bit machines */
		  };
struct s_uv {
		double ut;
		double az;
		double el;
		double ha;
		double uv[(NUM_SCOPES*(NUM_SCOPES-1))/2][2];
		char uv_names[(NUM_SCOPES*(NUM_SCOPES-1))/2][10];
	    };

#define ASTROMOD_UV_DATA		(FIRST_ASTROMOD_MESSAGE+8)

/* Client->Server or reverse, set observing wavelength in microns,data double */

#define ASTROMOD_SET_OBS_WAVELENGTH	(FIRST_ASTROMOD_MESSAGE+9)

/* Client->Server, request wavelength in microns, no data */

#define ASTROMOD_GET_OBS_WAVELENGTH	(FIRST_ASTROMOD_MESSAGE+10)

/* 
 * Client->Server, set active scopes data char[NUM_DELAY_LINES]
 * Server->Client, response to ASTROMOD_GET_ACTIVE_SCOPES, same data.
 * These are the same as OPLE_GET_ACTIVE_CARTS and OPLE_SET_ACTIVE_CARTS.
 */

#define ASTROMOD_SET_ACTIVE_SCOPES	(FIRST_ASTROMOD_MESSAGE+11)

/* Client->Server request for which carts are active. */

#define ASTROMOD_GET_ACTIVE_SCOPES	(FIRST_ASTROMOD_MESSAGE+12)

#define LAST_ASTROMOD_MESSAGE           (ASTROMOD_GET_ACTIVE_CARTS)

/************************************************************************/
/* CLASSIC messages 2200-2299                                           */
/*									*/
/************************************************************************/

#define FIRST_CLASSIC_MESSAGE           2200

/* Client->Server Set bandpass frequency */

struct s_cbp_filter {
			float freq;
			float width;
		    };

#define CLASSIC_SET_BP_FILTER		(FIRST_CLASSIC_MESSAGE+0)

/* client->Server Set NIRO sample time in microseconds , data is an integer */

#define CLASSIC_SET_NIRO_SAMPLE		(FIRST_CLASSIC_MESSAGE+1)

/* Client->Server Set NIRO wait time in microseconds , data is an integer */

#define CLASSIC_SET_NIRO_WAIT		(FIRST_CLASSIC_MESSAGE+2)

/* Clilent->Server Set NIRO mode, argument an integer, one of */

#ifndef NIRO_NORM
#define NIRO_NORM               0
#define NIRO_SCAN               1
#define NIRO_ASYN               2
#define NIRO_PICT               3
#define NIRO_SYNC               4
#endif

#define CLASSIC_NIRO_MODE		(FIRST_CLASSIC_MESSAGE+3)

/*
 * Client->Server, request current NIRO mode, no data
 * Server->Client, result of NIRO mode request, data integer
 */

#define CLASSIC_GET_NIRO_MODE		(FIRST_CLASSIC_MESSAGE+4)

/* Client->Server, request NIRO resync */

#define CLASSIC_SYNC_NIRO		(FIRST_CLASSIC_MESSAGE+5)

/* 
 * Client->Server, request data collection.
 * Data type depends on current NIRO mode.
 * For PICT mode the data is struct s_data_pict, same as irimage.
 * For NORM mode the data is
 */


struct s_cdata_norm {
			int	width;	/* mS Default 50 */
			char	skip_save; /* Default FALSE */
			char	skip_display; /* Default FALSE */
			char	skip_low_weight; /* Save only good scans? */
			char	use_target_memory; /* Guess? */
			char	display[256];  /* Must be something */
			char	beam5[256];	/* Which scope is beam 3? */
			char	beam6[256];	/* Which scope is beam 4? */
			char	obj_type[80];	/* Type of object */
		   };
			
#define CLASSIC_COLLECT_DATA		(FIRST_CLASSIC_MESSAGE+6)

/* 
 * Server->Client for sending status on PICT mode display data is
 * s_pict_status, as defined for irimage.
 */

#define CLASSIC_PICT_STATUS	     	(FIRST_CLASSIC_MESSAGE+7)

/* Server->client message to stop displaying pict status */

#define CLASSIC_PICT_STATUS_STOP     	(FIRST_CLASSIC_MESSAGE+8)
			
/* 
 * Server-> CLient for sending status data on NORM mode display data is 
 * s_norm_status as defined for irimage.
 */

#define CLASSIC_NORM_STATUS		(FIRST_CLASSIC_MESSAGE+9)
			
/* Server->client message to stop displaying norm status */

#define CLASSIC_NORM_STATUS_STOP     	(FIRST_CLASSIC_MESSAGE+10)
			
/* 
 * Client->Server open tracking socket data integer to say which OPLE
 */

#define CLASSIC_OPEN_TRACKING_SOCKET     (FIRST_CLASSIC_MESSAGE+11)

/* Client->Server for controlling the display socket, data s_socket_name */

#define CLASSIC_OPEN_DISPLAY_SOCKET      (FIRST_CLASSIC_MESSAGE+12)

/* Client->Server for requesting which pixels we are using on NIRO */

#define CLASSIC_GET_NIRO_PIXELS      	 (FIRST_CLASSIC_MESSAGE+13)

/* Either direction for controling setting NIRO pixels we use s_niro_pixels */

#define CLASSIC_SET_NIRO_PIXELS		 (FIRST_CLASSIC_MESSAGE+14)

/* 
 * Client->Server request DITHER_STATE message, no data 
 * Server->Client return dither state as single byte.
 */

#define CLASSIC_DITHER_STATE		(FIRST_CLASSIC_MESSAGE+15)

/* 
 * Client->Server, request NIRO sample time, no data.
 * Server should respond with a CLASSIC_SET_NIRO_SAMPLE message.
 */

#define CLASSIC_GET_NIRO_SAMPLE		(FIRST_CLASSIC_MESSAGE+16)

/* 
 * Client->Server, request NIRO wait time, no data.
 * Server should respond with a CLASSIC_SET_NIRO_NREADS message.
 */

#define CLASSIC_GET_NIRO_WAIT		(FIRST_CLASSIC_MESSAGE+17)

/* Client->Serevr, set beam combiner, data int */

#define CLASSIC_SET_BEAM_COMBINER	(FIRST_CLASSIC_MESSAGE+18)

/* Clear filter so it is wide open. */

#define CLASSIC_RESET_FILTER		(FIRST_CLASSIC_MESSAGE+19)

/* Client->Server for controlling the niro socket, data s_socket_name */

#define CLASSIC_OPEN_NIRO_SOCKET      (FIRST_CLASSIC_MESSAGE+20)

/* Client->Server for asking for power of two for number of reads, no data */

#define CLASSIC_GET_NIRO_NREADS	      (FIRST_CLASSIC_MESSAGE+21)

/*
 * Client->Server for setting number of reads, data struct
 * Server->Client response to CLASSIC_GET_NIRO_NREADS, data struct
 * s_niro_reads.
 */

#define CLASSIC_SET_NIRO_NREADS       (FIRST_CLASSIC_MESSAGE+22)

/* Client->Server for setting a standard NIRO type. Data integer, one of */

#ifndef NIRO_UNKNOWN
#define NIRO_UNKNOWN		-1
#define NIRO_1000HzNPIX2	0
#define NIRO_750HzNPIX2		1
#define NIRO_500HzNPIX2		2
#define NIRO_250HzNPIX2		3
#define NIRO_1000HzNPIX1	4
#define NIRO_750HzNPIX1		5
#define NIRO_500HzNPIX1		6
#define NIRO_250HzNPIX1		7
#define NUM_NIRO_STANDARD_TYPES	8
#endif

#define CLASSIC_SET_NIRO_STANDARD_TYPE	(FIRST_CLASSIC_MESSAGE+23)

/* Client->Server for changing the number of data scans data s_set_data_scans */

#define CLASSIC_SET_DATA_SCANS		(FIRST_CLASSIC_MESSAGE+24)
                                                                                
/* 
 * Client->Server Setting fast dither on or off, data unsigned char.
 * Server->Client MEssage to inform client of fast dither state, unsigned char
 */

#define CLASSIC_SET_FAST_DITHER		(FIRST_CLASSIC_MESSAGE+25)

/* Client->Server, request server to send CLASSIC_DITHER_STATE. */

#define CLASSIC_FAST_DITHER_STATE	(FIRST_CLASSIC_MESSAGE+26)

/* Client->Server set tracking cart, data an int. */

#define CLASSIC_SET_TRACKING_CART	(FIRST_CLASSIC_MESSAGE+27)

/* Client-> server set servo direction. data an int, only sign used. */

#define CLASSIC_SET_DIRECTION		(FIRST_CLASSIC_MESSAGE+28)

/* Client->server ask for current niro reset delay, no data */

#define CLASSIC_GET_NIRO_RESET_DELAY	(FIRST_CLASSIC_MESSAGE+29)

/* 
 * Server-> Client reply to previous, data integer
 * Client->Server set niro reset delay, data integer
 */

#define CLASSIC_SET_NIRO_RESET_DELAY	(FIRST_CLASSIC_MESSAGE+30)

/* Client->server ask for current niro reset time, no data */

#define CLASSIC_GET_NIRO_RESET_TIME	(FIRST_CLASSIC_MESSAGE+31)

/* 
 * Server-> Client reply to previous, data integer
 * Client->Server set niro reset time, data integer
 */

#define CLASSIC_SET_NIRO_RESET_TIME	(FIRST_CLASSIC_MESSAGE+32)

/* Client->Server Force dither to go ON. No data. */

#define CLASSIC_TURN_DITHER_ON		(FIRST_CLASSIC_MESSAGE+33)

/* Client->Server Force dither to go OFF. No data. */

#define CLASSIC_TURN_DITHER_OFF		(FIRST_CLASSIC_MESSAGE+34)

/* Client->Server Set new flexible dither, data as per s_flexible_dither */

#define CLASSIC_FLEXIBLE_DITHER		(FIRST_CLASSIC_MESSAGE+35)

/* Client->Server Set new feedback dither, data as per s_flexible_dither */

#define CLASSIC_FEEDBACK_DITHER		(FIRST_CLASSIC_MESSAGE+36)

/* CLient->Server, set offsets and dither. No data */

#define CLASSIC_CENTER_OFFSET		(FIRST_CLASSIC_MESSAGE+37)
#define CLASSIC_ZERO_OFFSET		(FIRST_CLASSIC_MESSAGE+38)
#define CLASSIC_ZERO_DITHER		(FIRST_CLASSIC_MESSAGE+39)

/*
 * Server-> Client: Send the filter used for data collection. Data is an array
 * of floats whose length must be determined from the length of the message.
 */

#define CLASSIC_FILTER			(FIRST_CLASSIC_MESSAGE+40)

/*
 * Server->Client: Send fringe difference signal. Data is an array of floats
 * whose length must be determined from the length of the message.
 */

#define CLASSIC_DIFFERENCE_SIGNAL	(FIRST_CLASSIC_MESSAGE+41)

/*
 * Server->Client: Send raw fringe signals. Data is an array of floats with the
 * two signals. The length of each array is determined as half the length 
 * implied by the length of the message, that is mess.length/(2*sizeof(float).
 */

#define CLASSIC_RAW_SIGNAL		(FIRST_CLASSIC_MESSAGE+42)

/*
 * Server->Client: Send fringe sum signals. Data is an array of floats with the
 * two signals. The length of each array is determined as half the length 
 * implied by the length of the message, that is mess.length/(2*sizeof(float).
 */

#define CLASSIC_SUM_SIGNAL		(FIRST_CLASSIC_MESSAGE+43)

/*
 * Server->Client: Send fringe power spectrum. Data is an array of floats whose
 * length must be determined from the length of the message.
 */

#define CLASSIC_PS_SIGNAL		(FIRST_CLASSIC_MESSAGE+44)

/*
 * Server->Client: Send mean envelope signal. Data is array of floats whose
 * length must be determined from the length of the message.
 */

#define CLASSIC_MEAN_ENVELOPE		(FIRST_CLASSIC_MESSAGE+45)

/*
 * Server->Client: Display the current fringe data.
 * Data is structure below and the two integers represent 
 * the array index of the two positions within the signal arrays defined
 * in the messages above.
 */

#define CLASSIC_DISPLAY_FRINGE		(FIRST_CLASSIC_MESSAGE+46)

struct s_display_fringe {
	int	target;
	int	fringe;
	char	auto_scale;
	char	normal_scale;
	char	dither_is_on;
	char	up;
	char	fringe_is_there;
	char	new_high_snr;
	char	increase_scale;
	char	decrease_scale;
		};

/*
 * Server->Client: Sends picture data. There are two pictures, both NxN sent
 * as a single long stream of floats. N to be calculated from the length of
 * the message.
 */

#define CLASSIC_DISPLAY_PICTURE		(FIRST_CLASSIC_MESSAGE+47)

/* Client->Server turn auto mode on or off. No data. */


#define CLASSIC_AUTO_ON			(FIRST_CLASSIC_MESSAGE+48)
#define CLASSIC_AUTO_OFF		(FIRST_CLASSIC_MESSAGE+49)

/* Client->Server: Request current status of envelope mean settings, no data. */

#define CLASSIC_GET_MEAN_ENV_PARAM	(FIRST_CLASSIC_MESSAGE+50)

/* Both ways: Send envelope mean paramters, data of structure below */

struct s_mean_env_param {
	char do_mean_env_calc;
	char env_shift_and_add;
	float env_memory;
			};

#define CLASSIC_SET_MEAN_ENV_PARAM	(FIRST_CLASSIC_MESSAGE+51)

#define LAST_CLASSIC_MESSAGE		CLASSIC_ENVELOPE_SIGNAL

/************************************************************************/
/* And now we define the signals that CLASSIC knows about.		*/
/************************************************************************/

/* These signals are used in the PICT display routine */

#define CLASSIC_SIGNAL_MAKE_BIAS		1
#define CLASSIC_SIGNAL_ZERO_BIAS		2
#define CLASSIC_SIGNAL_SHOW_PICTURE		3
#define CLASSIC_SIGNAL_AUTOSCALE		4
#define CLASSIC_SIGNAL_SHOW_DIFF		5
#define CLASSIC_SIGNAL_SHOW_FIT			6

/* These signals are used in the NORM display routine */

#define CLASSIC_SIGNAL_CLEAR			7
#define CLASSIC_SIGNAL_SCALE_UP			8
#define CLASSIC_SIGNAL_SCALE_DOWN		9
#define CLASSIC_SIGNAL_RECORD			10
#define CLASSIC_SIGNAL_HOLD			11
#define CLASSIC_SIGNAL_SEND			12
#define CLASSIC_SIGNAL_OFFSET_UP_1		13
#define CLASSIC_SIGNAL_OFFSET_DOWN_1		14
#define CLASSIC_SIGNAL_OFFSET_UP_500		15
#define CLASSIC_SIGNAL_OFFSET_DOWN_500		16
#define CLASSIC_SIGNAL_TOGGLE_TRACK_SERVO 	17
#define CLASSIC_SIGNAL_MIN_WEIGHT_UP		18
#define CLASSIC_SIGNAL_MIN_WEIGHT_DOWN		19
#define CLASSIC_SIGNAL_MAX_ERROR_DOWN		20
#define CLASSIC_SIGNAL_MAX_ERROR_UP		21
#define CLASSIC_SIGNAL_GAIN_DOWN		22
#define CLASSIC_SIGNAL_GAIN_UP			23
#define CLASSIC_SIGNAL_TOGGLE_DIRECTION 	24
#define CLASSIC_SIGNAL_TARG_UP_1		25
#define CLASSIC_SIGNAL_TARG_DN_1		26
#define CLASSIC_SIGNAL_TARG_UP_10		27
#define CLASSIC_SIGNAL_TARG_DN_10		28
#define CLASSIC_SIGNAL_POS_UP_SMALL		29
#define CLASSIC_SIGNAL_POS_DN_SMALL		30
#define CLASSIC_SIGNAL_POS_UP_LARGE		31
#define CLASSIC_SIGNAL_POS_DN_LARGE		32
#define CLASSIC_SIGNAL_SIG_DIFF			33
#define CLASSIC_SIGNAL_STOP			34
#define CLASSIC_SIGNAL_POS_UP_BIG		35
#define CLASSIC_SIGNAL_POS_DN_BIG		36
#define CLASSIC_SIGNAL_POS_UP_HUGE		37
#define CLASSIC_SIGNAL_POS_DN_HUGE		38
#define CLASSIC_SIGNAL_USE_FILTER		39
#define CLASSIC_SIGNAL_NORMAL_SCALE		40
#define CLASSIC_SIGNAL_AUTO_MODE		41
#define CLASSIC_SIGNAL_READY_FOR_DISPLAY	42

/* This is used in several places */

#define CLASSIC_SIGNAL_EXIT		50

/************************************************************************/
/* BUNKER_CLOCK based messages 2300-2399				*/
/************************************************************************/

#define FIRST_BUNKER_CLOCK_MESSAGE	2300

/* All Client->Server and calls routines with no arguments */

#define BUNKER_CLOCK_START		(FIRST_BUNKER_CLOCK_MESSAGE+0)
#define BUNKER_CLOCK_STOP		(FIRST_BUNKER_CLOCK_MESSAGE+1)
#define BUNKER_CLOCK_RESET_GPS		(FIRST_BUNKER_CLOCK_MESSAGE+2)

/* 
 * Client->Server requesting a sync operation on a particular process 
 *  Data is the name of the process
 */

#define BUNKER_CLOCK_MSYNC		(FIRST_BUNKER_CLOCK_MESSAGE+3)

#define LAST_BUNKER_CLOCK_MESSAGE	BUNKER_CLOCK_MSYNC

/************************************************************************/
/* VEGA_PERISOPE based messages 2400-2499                               */
/************************************************************************/

#define FIRST_VEGA_PERISCOPE_MESSAGE    2400

/* Client -> Server no data */

#define VEGA_PERISCOPE_RESET            (FIRST_VEGA_PERISCOPE_MESSAGE+0)
#define VEGA_PERISCOPE_UP               (FIRST_VEGA_PERISCOPE_MESSAGE+1)
#define VEGA_PERISCOPE_DOWN             (FIRST_VEGA_PERISCOPE_MESSAGE+2)

/*
 * Client->Server, request status, no data
 * Server->Client, status, data int.
 * If positive or zero result is status. If negative it's an error.
 */

#define VEGA_PERISCOPE_STATUS           (FIRST_VEGA_PERISCOPE_MESSAGE+3)

#define LAST_VEGA_PERISCOPE_MESSAGE     VEGA_PERISCOPE_STATUS

/************************************************************************/
/* PAVO messages 2500-2599                                              */
/************************************************************************/

#define FIRST_PAVO_MESSAGE              2500

/* These will have to be prefixed with "PAVO" if they cause any problems.
   Hopefully these variables won't change much, so are fine as #defines...*/
#define N_BASELINES 3
#define N_TELS 3

/* The various PAVO beams. */
#define Z_LENSLET 1
#define Z_MASKS   2
#define Z_B1_L2_X 3
#define Z_B1_L2_Y 4
#define Z_B2_L2_X 5
#define Z_B2_L2_Y 6
#define Z_B3_L2_X 7
#define Z_B3_L2_Y 8
#define Z_B1_M1_Y 9
#define Z_B1_M1_X 10
#define Z_B2_M1_Y 11
#define Z_B2_M1_X 12
#define Z_B3_M1_Y 13
#define Z_B3_M1_X 14
#define Z_B1_BSPLIT 15
#define Z_B2_BSPLIT 16
#define NUM_ZABER 16
#define NUM_ZABER_FIXED 4
#define Z_NBEAMS 3

/* Some Special defines for about which settings are good for data. */
#define LENSLET1 1
#define LENSLET2 1
#define DEFAULT_MASK 3
#define MASKS_OUT 2

/* PAVO display type defines. */
#define PAVO_DISPLAY_RAW     1
#define PAVO_DISPLAY_UNDISTORT   3
#define PAVO_DISPLAY_PC          4
/* Just like RAW, except a zoomed central box */
#define PAVO_DISPLAY_ZOOM    5 

/* Client->Server: Request Zaber status. No data 
   Server->Client: Return  Zaber status. Return a s_pavo_zaber_status */
#define PAVO_ZABER_STATUS               (FIRST_PAVO_MESSAGE+0)

struct s_pavo_zaber_status {
  char current_fixed_position[NUM_ZABER+1];
  int current_position[NUM_ZABER+1];
};

/* Goto fixed or relative positions */
#define PAVO_ZABER_GOTO_FIXED           (FIRST_PAVO_MESSAGE+1)
#define PAVO_ZABER_MOVE_REL             (FIRST_PAVO_MESSAGE+2)

struct s_pavo_zaber_goto {
  char unit;
  int position;
};

/* Client->Server. No data. Write positions in the standard file name. */
#define PAVO_ZABER_WRITE_DEFAULT        (FIRST_PAVO_MESSAGE+3)

/* Client->Server. Zero all motors. Returns zaber_status */
#define PAVO_ZABER_ZERO_ALL             (FIRST_PAVO_MESSAGE+4)

/* Server->Client Here is an image. Short xwidth, short ywidth
 then the compressed data */
/* Client->Server Please give me an image.*/
#define PAVO_IMAGE                      (FIRST_PAVO_MESSAGE+5)

/* Client->Server: Change the filter.*/
/* Server->Client: Report the current filter status. */
#define PAVO_FILTER                     (FIRST_PAVO_MESSAGE+6)

/* Client->Server: Change readout to VIDEO mode.*/
#define PAVO_VIDEO                      (FIRST_PAVO_MESSAGE+7)

/* Client->Server: Change readout to ACQUIRE mode.*/
#define PAVO_ACQUIRE                    (FIRST_PAVO_MESSAGE+8)

/* Client->Server: Change display mode.*/
#define PAVO_DISP_MODE                  (FIRST_PAVO_MESSAGE+9)

/* Client->Server: Start acquiring data.*/
#define PAVO_CAMERA_GO                  (FIRST_PAVO_MESSAGE+10)

/* Client->Server: Start acquiring data.*/
#define PAVO_CAMERA_STOP                (FIRST_PAVO_MESSAGE+11)

/* Client->Server: Start acquiring data.*/
#define PAVO_TOGGLE_SAVING              (FIRST_PAVO_MESSAGE+12)

/* Client->Server: Start acquiring data.*/
#define PAVO_TOGGLE_BACKGROUND          (FIRST_PAVO_MESSAGE+13)

/* Client->Server: Start acquiring data.*/
#define PAVO_TOGGLE_FOREGROUND          (FIRST_PAVO_MESSAGE+14)

/* Client->Server: Request camera/saving status */
/* Server->Client: Report camera/saving status */
/* Note that MJI wants to keep this really simple, with all the complex
   stuff in the server itself. */
struct s_pavo_camera_status {
  int are_we_saving;
  int camera_on_off;
  int back_on_off;
  int fore_on_off;
  float positions[N_TELS];
  float rates[N_TELS];
  int em_gain;
  int display_numavg;
  int auto_save;
};
#define PAVO_CAMERA_STATUS              (FIRST_PAVO_MESSAGE+15)

#define GD_DFT_SIZE 41 
struct s_display_gdplot {
  float data[N_BASELINES][GD_DFT_SIZE];
};

/* Server->Client Here is some group-delay DFTs to plot. */
/* Client->Server Please give me some group delay DFTs to plot.*/
#define PAVO_DISPLAY_GDPLOT             (FIRST_PAVO_MESSAGE+16)

/* Client->Server Start new Fringe search.*/
struct s_pavo_fringe_search {
  int tel;
  int offset;
  int rate;
};
#define PAVO_FRINGE_LINEAR_SEARCH       (FIRST_PAVO_MESSAGE+17)

/* Client->Server: Set the number of frames to average.*/
/* Server->Client: Report the number of frames being averaged.*/
#define PAVO_DISP_AVERAGE               (FIRST_PAVO_MESSAGE+18)

/* Client->Server: Set the emccd gain.*/
/* Server->Client: Report an change in the emccd gain.*/
#define PAVO_EMCCD_GAIN            (FIRST_PAVO_MESSAGE+19)

/* Client->Server: Synchronize the PAVO gain */
#define PAVO_PSYNC		   (FIRST_PAVO_MESSAGE+20)

/* Client->Server: Toggle the auto save feature.*/
#define PAVO_TOGGLE_AUTOSAVE       (FIRST_PAVO_MESSAGE+21)

/* Client->Server: Open a connection to one of the oples.*/
/* data: an "int". 1 for ople_1, 2 for ople_2 etc.*/
#define PAVO_REOPEN_OPLE           (FIRST_PAVO_MESSAGE+22)

#define LAST_PAVO_MESSAGE               PAVO_REOPEN_OPLE

/************************************************************************/
/* OBSERVE messages 2600-2699                                           */
/************************************************************************/
#define FIRST_OBSERVE_MESSAGE           2600

/* Client->Server: No data, ready to search for fringes */

#define OBSERVE_FRINGE_SEARCH           (FIRST_OBSERVE_MESSAGE+0)

/* Server->Client: Found fringes (on all baselines) */

#define OBSERVE_FRINGE_FOUND            (FIRST_OBSERVE_MESSAGE+1)

/* Client->Server: Instruct the server to passively acquire data */

#define OBSERVE_FRINGE_DATA_START       (FIRST_OBSERVE_MESSAGE+2)

/*
 * Server->Client: Fringe data has finished being acquired. 
 * Client->Server: Please stop taking data (and do a shutter sequence 
 * 			or whatever)
 */

#define OBSERVE_FRINGE_DATA_DONE        (FIRST_OBSERVE_MESSAGE+3)

/*
 * Client->Server: Instruct the fringe-tracking server to stop 
`* 			fringe-tracking
 * Server->Client: Let the client know that we are no longer 
 * 			tracking the fringe.
 */

#define OBSERVE_FRINGE_TRACK_STOP       (FIRST_OBSERVE_MESSAGE+4)

/* Client->Server: Tell which OPLE server it should use. Data Integer */

#define OBSERVE_OPLE_NUMBER		(FIRST_OBSERVE_MESSAGE+5)

/* Client->Server: Star has been acquired. No data. */

#define OBSERVE_STAR_ACQUIRED		(FIRST_OBSERVE_MESSAGE+6)

/* Client->Server: SPectral calibration started */

#define OBSERVE_SPECTRAL_CALIBRATION_START	(FIRST_OBSERVE_MESSAGE+7)

/* Client->Server: SPectral calibration finished */

#define OBSERVE_SPECTRAL_CALIBRATION_FINISHED	(FIRST_OBSERVE_MESSAGE+8)

/* CLient->Server: Abort */

#define OBSERVE_ABORT			(FIRST_OBSERVE_MESSAGE+9)

#define LAST_OBSERVE_MESSAGE            OBSERVE_ABORT        

/************************************************************************/
/* IRIS messages 2700-2799                                              */
/************************************************************************/

#define FIRST_IRIS_MESSAGE           2700

/* Client->Server: Send a command to the server. No data */

#define IRIS_COMMAND_OPEN		(FIRST_IRIS_MESSAGE+0)
#define IRIS_COMMAND_CLOSE		(FIRST_IRIS_MESSAGE+1)
#define IRIS_COMMAND_BEAM		(FIRST_IRIS_MESSAGE+2)
#define IRIS_COMMAND_DECREMENT		(FIRST_IRIS_MESSAGE+3)
#define IRIS_COMMAND_INCREMENT		(FIRST_IRIS_MESSAGE+4)
#define IRIS_COMMAND_STATUS		(FIRST_IRIS_MESSAGE+5)
#define IRIS_COMMAND_STOP		(FIRST_IRIS_MESSAGE+6)

/* Server->Client: Send the current status, Data int */

#define IRIS_STATUS_UNKNOWN		0x00
#define IRIS_STATUS_OPENING		0x01
#define IRIS_STATUS_CLOSING		0x02
#define IRIS_STATUS_HOMING		0x03
#define IRIS_STATUS_APP_BEAM		0x04
#define IRIS_STATUS_OPENING_1		0x05
#define IRIS_STATUS_OPENING_2		0x06
#define IRIS_STATUS_STOPPED		0x07
#define IRIS_STATUS_FULLY_OPEN		0x08
#define IRIS_STATUS_FULLY_CLOSED	0x09
#define IRIS_STATUS_BEAM_SIZE		0x0A
#define IRIS_STATUS_FIRST_EDGE		0x0B
#define IRIS_STATUS_DECREMENTING	0x0C
#define IRIS_STATUS_INCREMENTING	0x0D
#define IRIS_STATUS_AT_LIMIT		0x0E

#define IRIS_STATUS_MESSAGE          (FIRST_IRIS_MESSAGE+7)

#define LAST_IRIS_MESSAGE            IRIS_STATUS_MESSAGE

/************************************************************************/
/* NIRO messages 2800-2899                                             */
/************************************************************************/

#define FIRST_NIRO_MESSAGE           2800

/*
 * Server->Client: Sends output in two pixl mode. Data structure 
 * Unless communications are too slow, I don't think we'll use this.
 */

struct s_niro_two_pixels {
	float     dither_position;
	int  time_stamp; /* Will be long int on DOS */
	short int pixel_1;
	short int pixel_2;
		};

#define NIRO_TWO_PIXEL        (FIRST_NIRO_MESSAGE+0)

/* Server->Client: Sends output in three pixels. Data following structure */

struct s_niro_three_pixels {
	float     dither1_position;
	float     dither2_position;
	int  time_stamp; /* Will be long int on DOS */
	short int pixel_1;
	short int pixel_2;
	short int pixel_3;
		};

#define NIRO_THREE_PIXEL      (FIRST_NIRO_MESSAGE+1)

/*
 * Client->Server Instruct server to be busy or not, data char TRUE or FALSE.
 * TRUE will incriment the busy counter, FALSE will decriment it
 * Server-> Client, inform client of current busy state, data short int returns
 * busy count.
 * When busy NIRO will only respond to this message, and some others like
 * starting and stoping sending data yet to be determined.
 */

#define NIRO_BUSY               (FIRST_NIRO_MESSAGE+2)

/* Client->Server reuest a NIRO_BUSY message, no data */

#define IS_NIRO_BUSY            (FIRST_NIRO_MESSAGE+3)

/* Client->Server, set dither pattern, data in structure below */

struct s_niro_set_dither {
			float	lambda;	/* um */
			float	freq;	/* Hz */
			float	amplitude; /* um */
			short int	timing_offset;
			short int	dither; /* 0,1,2,3 */
		    };

#define NIRO_SET_DITHER            (FIRST_NIRO_MESSAGE+4)

/* Client->Server, turn dither on/off. Data short int 0-3 to set which dither */

#define NIRO_DITHER_ON             (FIRST_NIRO_MESSAGE+5)
#define NIRO_DITHER_OFF            (FIRST_NIRO_MESSAGE+6)

/* Client->Server - set dither offset, data in structure below */

struct s_niro_dither_offset {
				short int dither; /* 0,1,2 or 3 */
				short int offset; /* 0-2^15 */
			    };

#define NIRO_DITHER_OFFSET         (FIRST_NIRO_MESSAGE+7)

/*
 * Client->Server request dither status, data short int dither you want.
 * Server->Client Send dither status, data in structure below.
 */

struct s_niro_dither_status {
			float	lambda;	/* um */
			float	freq;	/* Hz */
			float	amplitude; /* um */
			float	current_position;
			short int position_on;
			short int dither_on;
			short int dither_offset;
			short int current_value;
			short int current_demand;
			short int dither;
			};

#define NIRO_DITHER_STATUS         (FIRST_NIRO_MESSAGE+8)

/* Client->Server arm the dither, data short int dither to arm. */

#define NIRO_ARM_DITHER         (FIRST_NIRO_MESSAGE+9)

/* Client->Server, turn position on/off. Data short int 0-3 set which dither */

#define NIRO_POSITION_ON             (FIRST_NIRO_MESSAGE+10)
#define NIRO_POSITION_OFF            (FIRST_NIRO_MESSAGE+11)

/*
 * Client-Server, request some picture data, data in form of structure below.
 * Server->Client, picture data. Data string of unsigned short ints. The client
 * presummably knows how many pixels to expect, and can at least know the total
 * number of data points it has received.
 */

struct s_niro_picture {
                        short int firstCol;
                        short int lastCol;
                        short int firstRow;
                        short int lastRow;
                        short int nreads;
                        short int nwaits;
                      };

#define NIRO_PICTURE            (FIRST_NIRO_MESSAGE+12)

/* Client->Server, set unit number, data short int (CLIMB_0 or CLIMB_1) */

#define CLIMB_1                 0
#define CLIMB_2                 1
#define CLASSIC                 CLIMB_2

#define NIRO_UNIT_NUMBER	(FIRST_NIRO_MESSAGE+13)

/* Client->Server, enable reading, no data, uses unit number already saved. */

#define NIRO_ENABLE_READ	(FIRST_NIRO_MESSAGE+14)

/* Client->Server, disable reading, no data, uses unit number already saved. */

#define NIRO_DISABLE_READ	(FIRST_NIRO_MESSAGE+15)

/* CLient->Server ask for current NIRO setup, no data */

#define NIRO_GET_NIRO_SETUP	(FIRST_NIRO_MESSAGE+16)

/* Both ways, set NIRO setup, data in struct */

struct s_niro_setup {
                float     frames_per_second;
                short int col1;
                short int col2;
                short int col3;
                short int col4;
                short int row1;
                short int row2;
		short int row3;
                short int nreads;
                short int nloops;
                short int nwaits;
		short int reset_delay;
                short int reset_interval;
                short int nresets;
		short int dither_delay;
		char read_mode;
                char read_climb_1;
                char read_climb_2;
                        };


#define NUM_READOUT_MODES               2
#define NIRO_DESTRUCTIVE_READ           0
#define NIRO_NONDESTRUCTIVE_READ        1

#define NIRO_SET_NIRO_SETUP	(FIRST_NIRO_MESSAGE+17)

/* Client->Server set niro readout method, data char and one of these macros */

#define NIRO_SET_READOUT_MODE           (FIRST_NIRO_MESSAGE+18)

/* Client->Server, request mode, no data. Server should send mode back */

#define NIRO_GET_MODE           	(FIRST_NIRO_MESSAGE+19)

/* Both ways set NIRO mode, data char and one of the macros below */

#define NUM_NIRO_MODES		3
#define NIRO_IDLE               0
#define NIRO_CONTINUOUS         1
#define NIRO_WITH_DITHER        2

#define NIRO_SET_MODE           	(FIRST_NIRO_MESSAGE+20)

/* 
 * Client->Server Request 3 pixel data, no data in message 
 * Response should be a NIRO_THREE_PIXELS message.
 */

#define NIRO_GET_THREE_PIXEL		(FIRST_NIRO_MESSAGE+21)

/* Both: set how and if the data is sent. Data char and one of: */

#define NUM_NIRO_SEND_MODES 3
#define NIRO_NO_SEND	0
#define NIRO_SEND_CONT	1
#define NIRO_SEND_SCAN	2

#define NIRO_SET_SEND_MODE		(FIRST_NIRO_MESSAGE+22)

/* Client -> Server ask for current send mode, no data. */

#define NIRO_GET_SEND_MODE		(FIRST_NIRO_MESSAGE+23)

/* Server->Client sends full scan: data array of three_pixels varying size. */

#define NIRO_SCAN_DATA			(FIRST_NIRO_MESSAGE+24)

/* Server->Client no data, let's client a scan is complete in continuous mode.*/

#define NIRO_SCAN_COMPLETE              (FIRST_NIRO_MESSAGE+25)

/* 
 * Client->Server request one of the standard readout types defined below.
 * Note that it isn't always possible to do what NIRO has been asked to do
 * so it will pick the cloest thing it can. The Client must work out how well
 * thing went. Data integer and one of those below:
 * Server->Client: let client know which mode we are in. UNKOWN used if 
 * things fail.
 */

#ifndef NIRO_READOUT_UNKNOWN
#define NIRO_READOUT_UNKNOWN    -1
#define NIRO_READOUT_FASTEST    0
#define NIRO_READOUT_2000Hz     1
#define NIRO_READOUT_1000Hz     2
#define NIRO_READOUT_750Hz      3
#define NIRO_READOUT_500Hz      4
#define NIRO_READOUT_250Hz      5
#define NIRO_READOUT_125Hz      6
#define NUM_NIRO_READOUT_MODES  7
#endif

#define NIRO_SET_STANDARD_READOUT               (FIRST_NIRO_MESSAGE+26)

/* Client->Server set FTS mode on or off. Data unsigned char */

#define NIRO_SET_FTS                            (FIRST_NIRO_MESSAGE+27)

#define LAST_NIRO_MESSAGE               NIRO_SET_FTS

/************************************************************************/
/* CLIMB messages 2900-2999                                             */
/************************************************************************/

#define FIRST_CLIMB_MESSAGE           2900

/* Client->Server try and reopen the NIRO socket, no data */

#define CLIMB_REOPEN_NIRO_SOCKET	(FIRST_CLIMB_MESSAGE+0)

/* 
 * Client->Server, turn position on/off. Data short int 0,1 set which dither 
 * Server->Client, inform client of position status, Data same
 */

#define CLIMB_POSITION_ON             (FIRST_CLIMB_MESSAGE+1)
#define CLIMB_POSITION_OFF            (FIRST_CLIMB_MESSAGE+2)

/* 
 * Client->Server, turn position on/off. Data short int 0,1 set which dither
 * Server->Client, inform client of dither status, Data same
 */

#define CLIMB_DITHER_ON             (FIRST_CLIMB_MESSAGE+3)
#define CLIMB_DITHER_OFF            (FIRST_CLIMB_MESSAGE+4)

/* 
 * Client->Server, request names of dither mirrors. No data
 * Server->Client, sending names of dither mirrors. Data s_names.
 */

#define CLIMB_DITHER_NAMES            (FIRST_CLIMB_MESSAGE+5)

/* Client->Server, request current position and dither status. No data */

#define CLIMB_DITHER_STATUS		(FIRST_CLIMB_MESSAGE+6)

/* Client->Server set a new dither pattern. Data struct s_climb_new_dither */

struct s_climb_new_dither {
			float	lambda;	/* um */
			float	freq;	/* Hz */
			float	amplitude; /* um */
			short int dither; /* 0 or 1 */
		    };

#define CLIMB_NEW_DITHER	(FIRST_CLIMB_MESSAGE+7)

/* Client->Server arm a dither, data short int (0, 1) */

#define CLIMB_ARM_DITHER	(FIRST_CLIMB_MESSAGE+8)

/* Client->Server set dither offset to middle or zero, data short int (0, 1) */

#define CLIMB_CENTER_OFFSET           (FIRST_CLIMB_MESSAGE+9)
#define CLIMB_ZERO_OFFSET             (FIRST_CLIMB_MESSAGE+10)

/* Server->Client send information about images being displayed, data struct */

struct s_climb_picture_status{
		float	average;
		float	frames_per_second;
		float	max;
		float	min;
		int num_frames;
		short int firstCol;
		short int firstRow;
		short int lastCol;
		short int lastRow;
		short int nreads;
		short int nwaits;
		short int maxx;
		short int minx;
		short int maxy;
		short int miny;
		short int unit_number;
			     };

#define CLIMB_PICTURE_STATUS		(FIRST_CLIMB_MESSAGE+11)

/* Server->Client Stop displaying pictures. no data */

#define CLIMB_PICTURE_STATUS_STOP	(FIRST_CLIMB_MESSAGE+12)

/* Client->Server Request the start of a picture movie, no data */

#define CLIMB_PICTURE_START	(FIRST_CLIMB_MESSAGE+13)

/* 
 * Server->Client, sending picture information. Data compressed 
 * array of floats
 */

#define CLIMB_DISPLAY_PICTURE	(FIRST_CLIMB_MESSAGE+14)

/* Client->Server ask for current NIRO setup, no data */

#define CLIMB_GET_NIRO_SETUP	(FIRST_CLIMB_MESSAGE+15)

/* Both ways, send current NIRO setup, data struct s_niro_setup. */

#define CLIMB_SET_NIRO_SETUP	(FIRST_CLIMB_MESSAGE+16)

/* Client->Server, request current NIRO mode, no data */

#define CLIMB_GET_NIRO_MODE	(FIRST_CLIMB_MESSAGE+17)

/* Both, set NIRO mode, data char */

#define CLIMB_SET_NIRO_MODE	(FIRST_CLIMB_MESSAGE+18)

/* Client->Server, request current NIRO send mode, no data */

#define CLIMB_GET_NIRO_SEND_MODE	(FIRST_CLIMB_MESSAGE+19)

/* Both, set NIRO send mode, data char */

#define CLIMB_SET_NIRO_SEND_MODE	(FIRST_CLIMB_MESSAGE+20)

/* Both, set NIRO read mode, data char */

#define CLIMB_SET_NIRO_READ_MODE	(FIRST_CLIMB_MESSAGE+21)

/* Client->Server Request the start of a picture area of interest, no data */

#define CLIMB_AREA_OF_INTEREST_START	(FIRST_CLIMB_MESSAGE+22)

/* Client->Server Request the start of a picture full chip, no data */

#define CLIMB_FULL_CHIP_START		(FIRST_CLIMB_MESSAGE+23)

/* Server->Client Tells client about the current alignment status */

struct s_climb_align_status {
			float	x;
			float	y;
			float	amp;
			float	x0;
			float	y0;
			float	fwhm;
			float	min;
			float	max;
			float	in;
			float	tot;
			short int pixel;
			short int firstCol;
			short int firstRow;
			short int lastCol;
			short int lastRow;
			      };

#define CLIMB_ALIGN_STATUS		(FIRST_CLIMB_MESSAGE+24)

/* Client->Server, request pixel names, no data. */

#define CLIMB_GET_PIXEL_NAMES		(FIRST_CLIMB_MESSAGE+25)

/* Server->Client, send pixel names, data in structure */

struct s_climb_pixel_names {
		char pixel1[32];
		char pixel2[32];
		char pixel3[32];
			   };

#define CLIMB_SET_PIXEL_NAMES		(FIRST_CLIMB_MESSAGE+26)

/* Client->server, request alignment data, data in struct */

struct s_climb_align {
		short int pixel;
		short int nreads;
		short int nwaits;
		     };

#define CLIMB_ALIGN_START   		    (FIRST_CLIMB_MESSAGE+27)

/* Server->Client, no data, stops alignment */

#define CLIMB_ALIGN_STOP   		    (FIRST_CLIMB_MESSAGE+28)

/* Client->Server open tracking socket data integer to say which OPLE */

#define CLIMB_OPEN_TRACKING_SOCKET           (FIRST_CLIMB_MESSAGE+29)

#define CLIMB_FILTER			(FIRST_CLIMB_MESSAGE+30)

/* Server-> Client for sending status data on NORM mode display data is */

struct	s_climb_two_pixel_status {
			float	position;
			float	snr_now;
			float	snr_max;
			float	snr_time;
			float   min_snr;
			float	v;
			float   pos;
			float   max_error;
			float   gain;
			float	fringetarg;
			float	sigmean;
			int	scans;
			char	hold;
			char	dither_is_on;
			char    track_servo_on;
			char	filename[128];
			char	comment[128];
		      };

#define CLIMB_TWO_PIXEL_STATUS		(FIRST_CLIMB_MESSAGE+31)
			
/* Server->client message to stop displaying two pixel data status */

#define CLIMB_TWO_PIXEL_STATUS_STOP 	(FIRST_CLIMB_MESSAGE+32)
			
/* Client->Server for changing the number of data scans data s_set_data_scans */

#define CLIMB_SET_DATA_SCANS		(FIRST_CLIMB_MESSAGE+33)
                                                                                
/*
 * Client->Server request beam types, no data
 * Server->Client send beam types with data in structure below.
 */

#define NUM_BEAM_TYPES	3

struct s_climb_beam_types{
				char beam[NUM_BEAM_TYPES][80];
			 };

#define CLIMB_BEAM_TYPES		(FIRST_CLIMB_MESSAGE+34)

/* Client->Server, request two pixel data. Data in struct */

struct s_climb_get_two_pixel_data {
		float	bp_freq;
		int	beams;
		char	skip_save;
		char 	skip_low_weight;
		char	use_target_memory;
		char	object_type[80];
				   };
		
#define CLIMB_GET_TWO_PIXEL_DATA	(FIRST_CLIMB_MESSAGE+35)


/*
 * Server->Client: Send fringe difference signal. Data is an array of floats
 * whose length must be determined from the length of the message.
 */

#define CLIMB_DIFFERENCE_SIGNAL		(FIRST_CLIMB_MESSAGE+36)

/*
 * Server->Client: Send raw signals. Data is a compressed array of 
 * floats with the two signals. The length of each array should match
 * the data_size element in the structure below.
 */

#define CLIMB_RAW_SIGNAL		(FIRST_CLIMB_MESSAGE+37)

/*
 * Server->Client: Send sum signals. Data is a compressed 
 * array of floats whose
 * length must be determined from the length of the message.
 */

#define CLIMB_SUM_SIGNAL		(FIRST_CLIMB_MESSAGE+38)

/*
 * Server->Client: Send fringe power spectrum. Data is a compressed 
 * array of floats whose
 * length must be determined from the length of the message.
 */

#define CLIMB_PS_SIGNAL			(FIRST_CLIMB_MESSAGE+39)

/*
 * Server->Client: Display the current fringe data.
 * Data is structure below and the two integers represent 
 * the array index of the two positions within the signal arrays defined
 * in the messages above.
 */

struct s_climb_display_fringe {
	float	dmin;
	float	dmax;
	int	target;
	int	fringe;
	char	auto_scale;
	char	dither_is_on;
	char	fringe_is_there;
	char	new_high_snr;
	char	do_mean_envelope;
        char    use_envelope_for_waterfall;
		};

#define CLIMB_DISPLAY_FRINGE		(FIRST_CLIMB_MESSAGE+40)

/* Client->Server set a new climb dither pattern. Data struct s_climb_dither */

struct s_climb_dither {
			float	lambda;	/* um */
			float	freq;	/* Hz */
			float	amplitude; /* um */
		    };

#define CLIMB_DITHER			(FIRST_CLIMB_MESSAGE+41)

/* Server-> Client for sending status data on Three pixel mode */

struct	s_climb_three_pixel_status {
			float	p1_mean;
			float	p2_mean;
			float	p3_mean;
			float	v_12;
			float	v_23;
			float	v_31;
			float	weight_12;
			float	weight_23;
			float	weight_31;
			float	target_12;
			float	target_23;
			float	target_31;
			float	pos_12;
			float	pos_23;
			float	pos_31;
			float   min_weight;
			float   max_error;
			float   gain;
			float	position_A;
			float	position_B;
			float	direction_A;
			float	direction_B;
			int	scans;
			int	tracking_cart_A;
			int	tracking_cart_B;
			char    track_servo_on;
			char	filename[128];
			char	comment[128];
		      };

#define CLIMB_THREE_PIXEL_STATUS	(FIRST_CLIMB_MESSAGE+42)
			
/* Server->Client stop with the status now. */

#define CLIMB_THREE_PIXEL_STATUS_STOP	(FIRST_CLIMB_MESSAGE+43)

/* Client->Server, request three pixel data. Data in struct */

struct s_climb_get_three_pixel_data {
		char	skip_save;
		char 	skip_low_weight;
		char	use_target_memory;
		char	object_type[80];
				   };
		
#define CLIMB_GET_THREE_PIXEL_DATA	(FIRST_CLIMB_MESSAGE+44)

/* 
 * Client->Server request one of the standard readout types defined above
 * as part of NIRO messages. Data integer and one of those below:
 * Server->Client: let client know which mode we are in. UNKOWN used if 
 * things fail.
 */

#define CLIMB_SET_STANDARD_READOUT      (FIRST_CLIMB_MESSAGE+45)

/* Client->Server: en/disable reading of the appropriate pixels. No data */

#define CLIMB_ENABLE_READ		(FIRST_CLIMB_MESSAGE+46)
#define CLIMB_DISABLE_READ		(FIRST_CLIMB_MESSAGE+47)

/* 
 * Client->Server: request a measurement of the current sample rate. No data.
 * Server->Client: send measured sample rate. Data float.
 */

#define CLIMB_SAMPLE_RATE		(FIRST_CLIMB_MESSAGE+48)

/*
 * Server->Client: Send fringe difference signal. Data is an array of floats
 * whose length must be determined from the length of the message.
 * This is for 3 pixel data and will contain three vectors for the
 * three fringe patterns 12/23/31. Data is compressed.
 */

#define CLIMB_3P_DIFFERENCE_SIGNAL	(FIRST_CLIMB_MESSAGE+49)

/*
 * Server->Client: Send raw signals. Data is a compressed array of 
 * floats with the two signals. The length of each array should match
 * the data_size element in the structure below.
 * This is for 3 pixel data and will contain three vectors for the
 * three fringe pixels 1,2 & 3. Data is compressed.
 */

#define CLIMB_3P_RAW_SIGNAL		(FIRST_CLIMB_MESSAGE+50)

/*
 * Server->Client: Send sum signals. Data is a compressed 
 * array of floats whose
 * length must be determined from the length of the message.
 * There will be three arrays for the three pixels.
 */

#define CLIMB_3P_SUM_SIGNAL		(FIRST_CLIMB_MESSAGE+51)

/*
 * Server->Client: Send fringe power spectrum. Data is a compressed 
 * array of floats whose
 * length must be determined from the length of the message.
 * THere will be three powerspectra here from the three output
 * pixels.
 */

#define CLIMB_3P_PS_SIGNAL		(FIRST_CLIMB_MESSAGE+52)

/*
 * Server->Client: Display the current fringe data.
 * Data is structure below and the two integers represent 
 * the array index of the two positions within the signal arrays defined
 * in the messages above.
 * This is for the 3 pixel version.
 */

struct s_climb_display_3p_fringe {
	float	frg_min;
	float	frg_max;
	int	target_12;
	int	target_23;
	int	target_31;
	int	fringe_12;
	int	fringe_23;
	int	fringe_31;
	char	fringe_is_there_12;
	char	fringe_is_there_23;
	char	fringe_is_there_31;
		};

#define CLIMB_DISPLAY_3P_FRINGE		(FIRST_CLIMB_MESSAGE+53)

/* Client->Server. Turn auto mode on or off, no data */

#define CLIMB_AUTO_MODE_ON		(FIRST_CLIMB_MESSAGE+54)
#define CLIMB_AUTO_MODE_OFF		(FIRST_CLIMB_MESSAGE+55)

/*
 * Server->Client: Send mean envelope signal. Data is array of compressed 
 * floats whose length must be determined from the length of the message.
 */

#define CLIMB_MEAN_ENVELOPE		(FIRST_CLIMB_MESSAGE+56)

/* Server->Client: Tells client BIAS frames are done. No data */

#define CLIMB_BIAS_COMPLETE		(FIRST_CLIMB_MESSAGE+57)

/* Both directions: Turn VME_DIRECT on or off. Data is an integer. */

#define CLIMB_VME_DIRECT		(FIRST_CLIMB_MESSAGE+58)

/* Client->Server: Request a CLIMB_VME message. No data */

#define CLIMB_GET_VME_DIRECT		(FIRST_CLIMB_MESSAGE+59)

/* Client->Server: Zero fringe tracking targets - no data  */

#define CLIMB_ZERO_FT			(FIRST_CLIMB_MESSAGE+60)

/* Server->Client send closure phase data in structure */

struct s_climb_closure_phase
	{
		float p2;
		float p3;
	};

#define CLIMB_CLOSURE_PHASE		(FIRST_CLIMB_MESSAGE+61)

#define LAST_CLIMB_MESSAGE          	CLIMB_CLOSURE_PHASE

/************************************************************************/
/* CLIMB signals 	                                                */
/************************************************************************/

#define CLIMB_SIGNAL_INCREASE_SIZE_ROW	1
#define CLIMB_SIGNAL_DECREASE_SIZE_ROW	2
#define CLIMB_SIGNAL_INCREASE_SIZE_COL	3
#define CLIMB_SIGNAL_DECREASE_SIZE_COL	4
#define CLIMB_SIGNAL_INCREASE_POS_ROW	5
#define CLIMB_SIGNAL_DECREASE_POS_ROW	6
#define CLIMB_SIGNAL_INCREASE_POS_COL	7
#define CLIMB_SIGNAL_DECREASE_POS_COL	8
#define CLIMB_SIGNAL_EXIT		9
#define CLIMB_SIGNAL_READY_FOR_DISPLAY	10
#define CLIMB_SIGNAL_INCREASE_NREADS	11
#define CLIMB_SIGNAL_DECREASE_NREADS	12
#define CLIMB_SIGNAL_INCREASE_NWAITS	13
#define CLIMB_SIGNAL_DECREASE_NWAITS	14

#define CLIMB_SIGNAL_MAKE_BIAS          15
#define CLIMB_SIGNAL_ZERO_BIAS          16
#define CLIMB_SIGNAL_SHOW_PICTURE       17
#define CLIMB_SIGNAL_SHOW_DIFF          18
#define CLIMB_SIGNAL_SHOW_FIT           19

#define CLIMB_SIGNAL_USE_FILTER		20
#define CLIMB_SIGNAL_CLEAR		21
#define CLIMB_SIGNAL_AUTOSCALE		22
#define CLIMB_SIGNAL_SCALE_UP		23
#define CLIMB_SIGNAL_SCALE_DOWN		24
#define CLIMB_SIGNAL_RECORD		25
#define CLIMB_SIGNAL_HOLD		26
#define CLIMB_SIGNAL_SEND		27
#define CLIMB_SIGNAL_TOGGLE_TRACK_SERVO	28
#define CLIMB_SIGNAL_MIN_WEIGHT_UP	29
#define CLIMB_SIGNAL_MIN_WEIGHT_DOWN	30
#define CLIMB_SIGNAL_MAX_ERROR_DOWN	31
#define CLIMB_SIGNAL_MAX_ERROR_UP	32
#define CLIMB_SIGNAL_GAIN_DOWN		33
#define CLIMB_SIGNAL_GAIN_UP		34
#define CLIMB_SIGNAL_TOGGLE_DIRECTION	35
#define CLIMB_SIGNAL_SIG_DIFF		36
#define CLIMB_SIGNAL_STOP		37
#define CLIMB_SIGNAL_POS_UP_SMALL	38
#define CLIMB_SIGNAL_POS_UP_LARGE	39
#define CLIMB_SIGNAL_POS_UP_BIG		40
#define CLIMB_SIGNAL_POS_UP_HUGE	41
#define CLIMB_SIGNAL_POS_DN_SMALL	42
#define CLIMB_SIGNAL_POS_DN_LARGE	43
#define CLIMB_SIGNAL_POS_DN_BIG		44
#define CLIMB_SIGNAL_POS_DN_HUGE	45
#define CLIMB_SIGNAL_TOGGLE_WATERFALL	46
#define CLIMB_SIGNAL_TARG_UP_1		47
#define CLIMB_SIGNAL_TARG_DN_1		48
#define CLIMB_SIGNAL_TARG_UP_10		49
#define CLIMB_SIGNAL_TARG_DN_10		50
#define CLIMB_SIGNAL_TOGGLE_DIRECTION_A	51
#define CLIMB_SIGNAL_TOGGLE_DIRECTION_B	52
#define CLIMB_SIGNAL_POS_UP_A		53
#define CLIMB_SIGNAL_POS_DN_A		54
#define CLIMB_SIGNAL_POS_UP_B		55
#define CLIMB_SIGNAL_POS_DN_B		56
#define CLIMB_SIGNAL_POS_SMALL		57
#define CLIMB_SIGNAL_POS_BIG		58
#define CLIMB_SIGNAL_POS_LARGE		59
#define CLIMB_SIGNAL_POS_HUGE		60
#define CLIMB_SIGNAL_AUTO_MODE		61

#define CLIMB_SIGNAL_DECREASE_DITHER	62

#define CLIMB_SIGNAL_CHANGE_FRINGE_TRACK_METHOD 63

#define CLIMB_SIGNAL_TOGGLE_NOISE_SUBTRACT 64

#define CLIMB_SIGNAL_SAVE_BLS_DATA	65

#define CLIMB_SIGNAL_TARG_UP_5		66
#define CLIMB_SIGNAL_TARG_DN_5		67

/************************************************************************/
/* APWHEELS messages 3000-3099                                          */
/************************************************************************/

#define FIRST_APWHEELS_MESSAGE           3000

/* Client->Server request wheel initialization, data int (0-5) */

#define APWHEELS_INITIALIZE		(FIRST_APWHEELS_MESSAGE+0)

/*
 * Client->Server: Request motion of a wheel to given aperture. Data stuct.
 * Server->Client: Report on a wheel's aperture. Data struct.
 * If aperture is negative, the system is not in a defined aperture position.
 */

struct s_apwheels_aperture {
				int	wheel;
				int	aperture;
			   };

#define APWHEELS_APERTURE		(FIRST_APWHEELS_MESSAGE+1)

/*
 * Client->Server: Request motion to a specific position. Data stuct.
 */

struct s_apwheels_position {
				int	wheel;
				int	position;
			   };

#define APWHEELS_POSITION		(FIRST_APWHEELS_MESSAGE+2)

/* Client->Server: Request an update on the positon of all wheels. No data. */

#define APWHEELS_UPDATE			(FIRST_APWHEELS_MESSAGE+3)

#define LAST_APWHEELS_MESSAGE          	APWHEELS_UPDATE

/************************************************************************/
/* CYLINDER messages 3100-3199                                          */
/************************************************************************/

#define FIRST_CYLINDER_MESSAGE		3100

/*
 * Server->Client: Push the current telemetry
 * data: s_cylinder_telemetry structure
 * Client->Server: Force push now. No data
 */

#define CYLINDER_STATUS			(FIRST_CYLINDER_MESSAGE+0)

struct s_cylinder_telemetry {
				double currpos;	/* current position in meters */
				int    mtrpos; /* current pos in motor counts */
				int    closestate;	/* is it closed? */
				int    active;
				int    driveok;
				int    cwlim;
				int    ccwlim;
				int    homedir;		/* homing direction */
				int    homestate;
				int    hmngstage; /* "smart" homing stage */
				int    cylinderid; /* cyl ID numeric code */
				int    currcomm;	/* current command */
				int    targetpos; /* current tgt in mtr cnts */
			    };
/*                 The hmngstage codes are as follows:
 *                 0	cylinder homing in default direction
 *                 1	cylinder gone too far, halting homing process
 *                 2	cylinder moving to near limit
 *                 3	cylinder found near limit
 *                 4	cylinder moved just off near limit
 *                 The cylinderid numeric code is as follows:
 *                 0	S1_Upper
 *                 1	S1_Lower
 *                 2	S2_Upper
 *                 3	S2_Lower
 *                 4	E1_Upper
 *                 5	E1_Lower
 *                 6	E2_Upper
 *                 7	E2_Lower
 *                 8	W1_Upper
 *                 9	W1_Lower
 *                 10	W2_Upper
 *                 11	W2_Lower
 *                 -1 means we don't know the cylinder
 *                 The currcomm codes are as follows:
 *                 0	no motion commanded
 *                 1	cylinder commanded to close
 *                 2	cylinder commanded to a preset
 *                 3	cylinder commanded to an arbitrary position
 *                 4	cylinder commanded to fully open position
 *                 5	cylinder commanded to back off of a limit
 *                 6	cylinder commanded to execute a homing sequence
 *                 7	cylinder stopped in mid-motion
 */

/*
 * Server->Client: Check for a stall condition.  Push the condition to the
 *		   the clients
 * data: integer, 1 for stall condition, 0 for normal
 */

#define CYLINDER_STALL_STATUS		(FIRST_CYLINDER_MESSAGE+1)

/*
 * Client->Server: Get the preset position numeric values.
 * data: array of doubles
 */

#define CYLINDER_GET_PRESETS		(FIRST_CYLINDER_MESSAGE+2)

/*
 * Client->Server: Stop a current motion command.
 * data: no data
 */

#define CYLINDER_STOP			(FIRST_CYLINDER_MESSAGE+3)

/*
 * Client->Server: Initiate a move to the closed position
 * data: no data
 */

#define CYLINDER_CLOSE			(FIRST_CYLINDER_MESSAGE+4)

/*
 * Client->Server: Initiate a move to the observing position
 * data: no data
 */

#define CYLINDER_OBSERVE		(FIRST_CYLINDER_MESSAGE+5)

/*
 * Client->Server: Initiate a move to a preset position
 * data: integer representing preset
 */

#define CYLINDER_PRESET_MOVE		(FIRST_CYLINDER_MESSAGE+6)

/*
 * Client->Server: Initiate a move to an absolute position
 * data: double representing position in meters to move from the closed position
 */

#define CYLINDER_ARBPOS_MOVE		(FIRST_CYLINDER_MESSAGE+7)

/*
 * Client->Server: Initiate a move to the fully opened position
 * data: no data
 */

#define CYLINDER_OPEN			(FIRST_CYLINDER_MESSAGE+8)

/*
 * Client->Server: Initiate a homing procedure
 * data: no data
 */

#define CYLINDER_HOME			(FIRST_CYLINDER_MESSAGE+9)

/* Clinet->Server Enable/Disable no data */

#define CYLINDER_ENABLE			(FIRST_CYLINDER_MESSAGE+10)
#define CYLINDER_DISABLE		(FIRST_CYLINDER_MESSAGE+11)

#define LAST_CYLINDER_MESSAGE		CYLINDER_DISABLE

/************************************************************************/
/* REFCAM messages 3200-3299                                            */
/************************************************************************/

#define FIRST_REFCAM_MESSAGE		3200

/* Client->Server, request an image of the reference camera. No data. */

#define REFCAM_REF			(FIRST_REFCAM_MESSAGE+0)

/* Server->Client, send reference camera status, data struct s_refcam_status */

struct s_refcam_status {
			float	maximum;
			float	pos_x;
			float	pos_y;
			float 	org_x;
			float	org_y;
			float	err_x;
			float	err_y;
			float	cur_x;
			float	cur_y;
			int	col;
			int	row;
			int	brightness;
			int	contrast;
			int 	zoom;
			int	mean_n;
			int	cross_hair;
			int	invert;
		    };

#define REFCAM_REF_STATUS		(FIRST_REFCAM_MESSAGE+1)

/* Server->Client, Tell client reference camera stuff is done. No data . */

#define REFCAM_REF_STOP			(FIRST_REFCAM_MESSAGE+2)

/* Client->Server, request an image of the reference camera. No data. */

#define REFCAM_CHAN			(FIRST_REFCAM_MESSAGE+3)

/* Server->Client, send reference camera status, data struct s_chancam_status */

struct s_chancam_status {
			float	maximum;
			int	col;
			int	row;
			int	brightness;
			int	contrast;
			int	show_blue_line;
		    };

#define REFCAM_CHAN_STATUS		(FIRST_REFCAM_MESSAGE+4)

/* Server->Client, Tell client chanel spectra camera stuff is done. No data . */

#define REFCAM_CHAN_STOP		(FIRST_REFCAM_MESSAGE+5)

/* Server->Client, Send compressed picture in bytes for reference. */

#define REFCAM_DISPLAY_REF		(FIRST_REFCAM_MESSAGE+6)

/* Server->Client, Send compressed matrix in floats for channel spectrum */

#define REFCAM_DISPLAY_CHAN		(FIRST_REFCAM_MESSAGE+7)

#define LAST_REFCAM_MESSAGE		REFCAM_DISPLAY_CHAN

/************************************************************************/
/* signals, mostly used for within functions.				*/
/************************************************************************/

#define REFCAM_SIGNAL_STOP		1
#define REFCAM_SIGNAL_CLEAR		2
#define REFCAM_SIGNAL_BRIGHT_UP		3
#define REFCAM_SIGNAL_BRIGHT_DOWN	4
#define REFCAM_SIGNAL_CONTRAST_UP	5
#define REFCAM_SIGNAL_CONTRAST_DOWN	6
#define REFCAM_SIGNAL_INVERT_COLOR	7
#define REFCAM_SIGNAL_SET_ORIGIN	8
#define REFCAM_SIGNAL_ZOOM_IN		9
#define REFCAM_SIGNAL_ZOOM_OUT		10
#define REFCAM_SIGNAL_CROSS_HAIR	11
#define REFCAM_SIGNAL_BIAS		14
#define REFCAM_SIGNAL_BLUE_LINE		15
#define REFCAM_SIGNAL_SAVE		16
#define REFCAM_SIGNAL_MEAN_UP		17
#define REFCAM_SIGNAL_MEAN_DOWN		18
#define REFCAM_SIGNAL_UP		19
#define REFCAM_SIGNAL_DOWN		20
#define REFCAM_SIGNAL_LEFT		21
#define REFCAM_SIGNAL_RIGHT		22
#define REFCAM_SIGNAL_ZERO_BIAS		23
#define REFCAM_SIGNAL_READY_FOR_DISPLAY	24

/************************************************************************/
/* VEGA messages 3300-3399                                              */
/************************************************************************/

#define FIRST_VEGA_MESSAGE		3300

#define LAST_VEGA_MESSAGE		FIRST_VEGA_MESSAGE

/************************************************************************/
/* FLUOR messages 3400-3499                                             */
/************************************************************************/

#define FIRST_FLUOR_MESSAGE		3400

/* Client->Server open tracking socket data integer to say which OPLE */

#define FLUOR_OPEN_TRACKING_SOCKET     (FIRST_FLUOR_MESSAGE+0)

/* CLient->Server attempts to open the NICMOS port. No data */

#define FLUOR_OPEN_NICMOS_PORT         (FIRST_FLUOR_MESSAGE+1)

/* Either direction. Set sample time. Data integer */

#define FLUOR_NICMOS_SET_SAMPLE	       (FIRST_FLUOR_MESSAGE+2)

/* Client->Server. Ask for the current sample. No data */

#define FLUOR_NICMOS_GET_SAMPLE	       (FIRST_FLUOR_MESSAGE+3)

/* Either direction. Set scan length. Data integer */

#define FLUOR_NICMOS_SET_LENGTH	       (FIRST_FLUOR_MESSAGE+4)

/* Client->Server. Ask for the current scan length No data */

#define FLUOR_NICMOS_GET_LENGTH	       (FIRST_FLUOR_MESSAGE+5)

/* Either direction. Set mode. Data unsigned char */

#define NUM_NICMOS_MODES 	  4
#define NICMOS_DESTRUCTIVE        0
#define NICMOS_NONDESTRUCTIVE     1
#define NICMOS_FRAME     	  2
#define NICMOS_CONTINUOUS    	  3

#define FLUOR_NICMOS_SET_MODE	       (FIRST_FLUOR_MESSAGE+6)

/* Client->Server. Ask for the current scan length No data */

#define FLUOR_NICMOS_GET_MODE	       (FIRST_FLUOR_MESSAGE+7)

/* Client->Server. Arm NICMOS. No data */

#define FLUOR_NICMOS_ARM	       (FIRST_FLUOR_MESSAGE+8)

/* Either direction. Turn communication on or off. Data unsigned char */

#define FLUOR_NICMOS_SET_ONOFF	       (FIRST_FLUOR_MESSAGE+9)

/* Client->Server. Ask for the current scan length No data */

#define FLUOR_NICMOS_GET_ONOFF	       (FIRST_FLUOR_MESSAGE+10)

/* Client->Server. flush NICMOS port. No data */

#define FLUOR_NICMOS_FLUSH	       (FIRST_FLUOR_MESSAGE+11)

/* Either direction. Set NICMOS spectrum size. Data unsigned char */

#define FLUOR_NICMOS_SET_SIZE	       (FIRST_FLUOR_MESSAGE+12)

/* Client->Server. Ask for the current spectrum size. No data */

#define FLUOR_NICMOS_GET_SIZE	       (FIRST_FLUOR_MESSAGE+13)

/* Either direction. Set NICMOS nreads. Data unsigned char */

#define FLUOR_NICMOS_SET_NREADS	       (FIRST_FLUOR_MESSAGE+14)

/* Client->Server. Ask for the current nreads. No data */

#define FLUOR_NICMOS_GET_NREADS	       (FIRST_FLUOR_MESSAGE+15)

/* Either direction. Set NICMOS nloops. Data unsigned char */

#define FLUOR_NICMOS_SET_NLOOPS	       (FIRST_FLUOR_MESSAGE+16)

/* Client->Server. Ask for the current nloops. No data */

#define FLUOR_NICMOS_GET_NLOOPS	       (FIRST_FLUOR_MESSAGE+17)

/* Server->Client send information about images being displayed, data struct */

struct s_fluor_picture_status{
		float	average;
		float	frames_per_second;
		float	max;
		float	min;
		float	max_value;
		float	min_value;
		float	sig1;
		float	sig2;
		float	photom1;
		float	photom2;
		float	frac_sig1;
		float	frac_sig2;
		float	frac_photom1;
		float	frac_photom2;
		int num_frames;
		short int firstCol;
		short int firstRow;
		short int lastCol;
		short int lastRow;
		short int nreads;
		short int sample;
		short int maxx;
		short int minx;
		short int maxy;
		short int miny;
			     };

#define FLUOR_PICTURE_STATUS		(FIRST_FLUOR_MESSAGE+18)

/* Server->Client Stop displaying pictures. no data */

#define FLUOR_PICTURE_STATUS_STOP	(FIRST_FLUOR_MESSAGE+19)

/* Client->Server Request the start of a picture movie, no data */

#define FLUOR_PICTURE_START		(FIRST_FLUOR_MESSAGE+20)

/* 
 * Server->Client, sending picture information. Data compressed 
 * array of floats
 */

#define FLUOR_DISPLAY_PICTURE		(FIRST_FLUOR_MESSAGE+21)

/* CLient->Server, set which beams we are using. Data int. */

#define FLUOR_BEAMS_34                0
#define FLUOR_BEAMS_56                1

#define FLUOR_SET_BEAMS			(FIRST_FLUOR_MESSAGE+22)

/* Client->Server, request data. Data in struct */

struct s_fluor_get_data {
		char	skip_save;
		char 	skip_low_weight;
		char	use_target_memory;
		char	object_type[80];
				   };
		
#define FLUOR_GET_DATA			(FIRST_FLUOR_MESSAGE+23)

/* Client->Server - set scan numbers data in structure */

struct s_fluor_set_scans {
		int data;
		int shutters;
		int darks;
			 };

#define FLUOR_SET_SCANS			(FIRST_FLUOR_MESSAGE+24)

/* Client->Server - turn Dither on/off, set to short or long scans - no data */

#define FLUOR_DITHER_ON			(FIRST_FLUOR_MESSAGE+25)
#define FLUOR_DITHER_OFF		(FIRST_FLUOR_MESSAGE+26)
#define FLUOR_DITHER_SHORT		(FIRST_FLUOR_MESSAGE+27)
#define FLUOR_DITHER_LONG		(FIRST_FLUOR_MESSAGE+28)
#define FLUOR_DITHER_ARM		(FIRST_FLUOR_MESSAGE+29)

#define LAST_FLUOR_MESSAGE		(FIRST_FLUOR_MESSAGE+29)


/************************************************************************/
/* FLUOR signals 	                                                */
/************************************************************************/

#define FLUOR_SIGNAL_EXIT			0
#define FLUOR_SIGNAL_READY_FOR_DISPLAY		1
#define FLUOR_SIGNAL_DECREASE_SAMPLE		2
#define FLUOR_SIGNAL_INCREASE_SAMPLE		3
#define FLUOR_SIGNAL_DECREASE_MAX		4
#define FLUOR_SIGNAL_INCREASE_MAX		5
#define FLUOR_SIGNAL_ZERO_BIAS			6
#define FLUOR_SIGNAL_MAKE_BIAS			7
#define FLUOR_SIGNAL_CAMERA_UP			8
#define FLUOR_SIGNAL_CAMERA_DOWN		9
#define FLUOR_SIGNAL_CAMERA_LEFT		10
#define FLUOR_SIGNAL_CAMERA_RIGHT		11

/************************************************************************/
/* DOME messages 3500-3599                                              */
/************************************************************************/

#define FIRST_DOME_MESSAGE		3500

#define NUM_AXIS                        2
#define AZ_AXIS                         0
#define EL_AXIS                         1
#define RA_AXIS                         0
#define DEC_AXIS                        1

/* Server -> clients. Sends current status. Uses Structure: */

#define DOME_HOMED			(1)
#define DOME_HOMING			(1<<1)
#define DOME_MOVING_CLOCKWISE		(1<<2)
#define DOME_MOVING_ANTICLOCKWISE	(1<<3)
#define DOME_MOVING			(1<<4)
#define DOME_STALLED			(1<<5)
#define DOME_IS_OPEN			(1<<6)
#define DOME_SERVO_ON			(1<<7)
#define SCOPE_SERVO_ENABLED		(1<<8)

struct s_dome_status {
        double  demand_position[NUM_AXIS];
        double  current_position[NUM_AXIS];
        double  current_velocity[NUM_AXIS];
        double  demand_ra_dec[NUM_AXIS];
        double  current_ra_dec[NUM_AXIS];
	float	dome_target;
	float	dome_position;
	int	dome_encoder;
	int	dome_status;
	int	drive_mode;
		};

#define DOME_STATUS	(FIRST_DOME_MESSAGE+0)

/* Client->Server: Try and find home. No data */

#define DOME_HOME		(FIRST_DOME_MESSAGE+1)

/* Client->Server: Move to a given position. Data float in degrees, 0 north */

#define DOME_POSITION		(FIRST_DOME_MESSAGE+2)

/* Client->Server: Open slit. No data */

#define DOME_OPEN		(FIRST_DOME_MESSAGE+3)

/* Client->Server: CLose slit. No data */

#define DOME_CLOSE		(FIRST_DOME_MESSAGE+4)

/* Client->Server: Set dome position. Data float in degrees */

#define DOME_INIT		(FIRST_DOME_MESSAGE+5)

/* Client->Server move the dome about or stop it. No data */

#define DOME_CLOCKWISE		(FIRST_DOME_MESSAGE+6)
#define DOME_ANTICLOCKWISE	(FIRST_DOME_MESSAGE+7)
#define DOME_STOP		(FIRST_DOME_MESSAGE+8)

#define DOME_REOPEN_DRIVERS	(FIRST_DOME_MESSAGE+9)

/* Both. Set mode of drive control. Data int. */

#define NUM_DRIVE_SERVO_MODES	3
#define DRIVE_SERVO_OFF		0
#define DRIVE_SERVO_MANUAL	1
#define DRIVE_SERVO_AUTO	2

#define DOME_SET_DRIVE_MODE	(FIRST_DOME_MESSAGE+10)

/* Client->Server set drive velocity, data structure */

struct s_drive_vel {
	double	velocity;
	int	drive;
		   };

#define DOME_SET_AXIS_VELOCITY	(FIRST_DOME_MESSAGE+11)

/* Client->Server Enable or Disable drive servo. */

#define DRIVE_ENABLE		(FIRST_DOME_MESSAGE+12)
#define DRIVE_DISABLE		(FIRST_DOME_MESSAGE+13)

/* Client->Server Initialize scope position. Data structure */

struct s_scope_init {
	double az;
	double el;
		    };

#define SCOPE_INIT		(FIRST_DOME_MESSAGE+14)

/* Client->Server Start recording data. Dta ainteger number of points */

#define DRIVE_RECORD		(FIRST_DOME_MESSAGE+15)

/* Server->Client Send data file name of recording. NULL terminated string */

#define DRIVE_RECORD_FILENAME  	(FIRST_DOME_MESSAGE+16)

/* Client->Server paddle changes - data char */

#ifndef PADDLE_STOP
#define PADDLE_STOP                0
#define PADDLE_AZ_INC              1
#define PADDLE_AZ_DEC              2
#define PADDLE_EL_INC              4
#define PADDLE_EL_DEC              8
#endif

#define DRIVE_PADDLE  		(FIRST_DOME_MESSAGE+17)

/* Client_Server ask for current servo parameters. Data int AZ_AXIS/EL_AXIS */

#define DRIVE_GET_SERVO_PARAMETERS (FIRST_DOME_MESSAGE+18)

/* Both ways, send drive servo parameters. Data structure */

struct s_drive_servo_parameters {
		double	sqrt_gain;
		double	tracking_gain;
		double	position_gain;
		double  damp;
		double	max_tracking_error;
		double  switch_velocity;
		double 	integrate_gain;
		double	sum_max;
		int	axis;
			};

#define DRIVE_SET_SERVO_PARAMETERS (FIRST_DOME_MESSAGE+19)

/* CLient->Server, set manual position data struct s_scope_init */

#define DRIVE_SET_MANUAL_POSITION (FIRST_DOME_MESSAGE+20)

/* Client->Server incriment manual position data struct s_scope_init */

#define DRIVE_INC_MANUAL_POSITION (FIRST_DOME_MESSAGE+21)

/* Client->Server Set paddle mode to velocity. Data double = velocity */

#define DRIVE_PADDLE_VELOCITY     (FIRST_DOME_MESSAGE+22)

/* Client->Server Set paddle mode to percent. Data double = 0-1 */

#define DRIVE_PADDLE_PERCENT      (FIRST_DOME_MESSAGE+23)

/* Client->Server set RA and DEC, data struct s_radec = see TELESCOPE_RADEC */

#define DRIVE_SET_RADEC         (FIRST_DOME_MESSAGE+24)

/* Client->Server Send scope to next position, no data */

#define DRIVE_NEXT              (FIRST_DOME_MESSAGE+25)

/* Client->Server Initialize on demand Ra/Dec. No data */

#define SCOPE_INIT_RADEC	(FIRST_DOME_MESSAGE+26)

/* Client-> Server - data single char ON and OFF See TELESCOPE_AUTODOME*/

#define DOME_AUTODOME		(FIRST_DOME_MESSAGE+27)

/* Client-> Server - no data */

#define DOME_STOW		(FIRST_DOME_MESSAGE+28)

/* CLient->Server - Set star by HD, data int */

#define DRIVE_SET_STAR_BY_HD	(FIRST_DOME_MESSAGE+29)

/*
 * client->server - Request Tpoints data. No data.
 * server->client - Send Tpoints data, data in struct
 */

struct s_tpoints {
	double	demand_az; /* Demand position of scope in radians */
	double  demand_el;
	double	stellar_az; /* Theoretical position of Star in radians */
	double  stellar_el;
		};
	
#define SCOPE_TPOINTS		(FIRST_DOME_MESSAGE+30)

/* Client->Server, intialize dome to telescope azimuth. No data */

#define DOME_INIT_TO_SCOPE	(FIRST_DOME_MESSAGE+31)

#define LAST_DOME_MESSAGE       (DOME_INIT_TO_SCOPE)

/************************************************************************/
/* These are not strictly speaking messages, but they are here so       */
/* all clients can see them. These define default ports to use for the  */
/* remote display programs and an increment to use if things do not     */
/* work.                                                                */
/* This stuff is largly obsolete now.					*/
/************************************************************************/

#define IRIMAGE_DISPLAY_PORT_START      3000
#define WOBBLE_DISPLAY_PORT_START       3001
#define S1_DISPLAY_PORT_START           3002
#define S2_DISPLAY_PORT_START           3003
#define E1_DISPLAY_PORT_START           3004
#define E2_DISPLAY_PORT_START           3005
#define W1_DISPLAY_PORT_START           3006
#define W2_DISPLAY_PORT_START           3007
#define IRIMAGE_NIRO_PORT_START      	3008
#define WOBBLE_REF_PORT_START      	3009
#define TIPTILT_DISPLAY_PORT_START     	3010
#define CLASSIC_DISPLAY_PORT_START     	3011
#define CLASSIC_NIRO_PORT_START      	3012

#define DISPLAY_PORT_INC                  20

#endif
